<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

if (!function_exists('exit_error')) {
    /**
     * エラー画面を表示して終了する
     *
     * @param string $html
     */
    function exit_error(string $html)
    {
        $content = <<<EOD
<!DOCTYPE html><html lang="ja"><head><meta charset="UTF-8"><title>MicroEngine Application Error</title><meta name="viewport" content="width=device-width,initial-scale=1">
<style>body{margin:0;padding:15px}h1{margin:0 0 15px;font-size:24px;color:#F70}article{background-color:#eee;padding:10px 15px}article+article{margin:20px 0 0}h2{border-bottom:1px solid #ccc;margin:0 0 10px;font-size:18px;padding:0 0 5px}pre{background-color:#f7f7f7;padding:10px;margin:0 0 5px;overflow-x:scroll}</style></head><body>
<header><h1>MicroEngine Application Error</h1></header>
{$html}
</body></html>
EOD;

        exit($content);
    }
}

// 定数定義
const INDEX_FILENAME = 'index';
const HTML_EXTENSION = '.html';

// 基本設定
ini_set('default_charset', 'UTF-8');

// 設定ファイルを読み込む
if (file_exists(__DIR__ . '/mecms_config.php')) {
    $mecmsConfig = require __DIR__ . '/mecms_config.php';
} else {
    $mecmsConfig = [
        'system_paths' => [
            'system',
            '../system',
            '../../system',
        ],
        'session_name' => 'MECMS_SESSID',
        'cookie_secure' => 0,
    ];
}

// セッション名を指定
$sessionName = $mecmsConfig['session_name'] ?? 'MECMS_SESSID';
session_name($sessionName);

// cookie_secure設定が有効な場合は、PHPのsession.cookie_secureを有効にする
if (isset($mecmsConfig['cookie_secure']) && $mecmsConfig['cookie_secure']) {
    ini_set('session.cookie_secure', true);
}
// セッションクッキーのhttpOnly属性は常にセットする
ini_set('session.cookie_httponly', true);

// セッションを開始する
session_start();

$systemPathPrefix = null;
foreach ($mecmsConfig['system_paths'] as $prefix) {
    if (file_exists($prefix . '/vendor/autoload.php')) {
        $systemPathPrefix = $prefix;
        break;
    }
}
if ($systemPathPrefix === null) {
    exit_error('The autoload file does not found!');
}

// htmlキャッシュの利用
$htmlCacheDir = $systemPathPrefix . '/storage/cache/html';
$htmlCacheFilePath = $htmlCacheDir . $_SERVER['REQUEST_URI'];
if (substr($htmlCacheFilePath, -1) === '/') {
    // スラッシュで終わっているときは、インデックスファイル名を追加する
    $htmlCacheFilePath .= INDEX_FILENAME . HTML_EXTENSION;
}
// 有効期限の確認
$cacheExpired = false;
$expirationDateFilePath = $htmlCacheDir . '/expiration_date.txt';
if (file_exists($expirationDateFilePath)) {
    $expirationUnixtime = file_get_contents($expirationDateFilePath);
    if ($expirationUnixtime && $expirationUnixtime <= time()) {
        // 有効期限を過ぎていれば、期限切れと判断する
        $cacheExpired = true;
    }
}
// ログイン状態を確認
$authenticated = $_SESSION['authenticated'] ?? false;
// 未ログイン状態で、キャッシュの有効期限がきれていない場合は対応するキャッシュファイルがあることを確認
if (!$authenticated && !$cacheExpired && is_file($htmlCacheFilePath)) {
    // キャッシュディレクトリ内のファイルであることを確認する
    $realHtmlCacheDir = realpath($htmlCacheDir);
    $realHtmlCacheFilePath = realpath($htmlCacheFilePath);
    if ($realHtmlCacheFilePath === false || strpos($realHtmlCacheFilePath, $realHtmlCacheDir) !== 0) {
        exit_error('Cache file does not exist.');
    }
    // キャッシュディレクトリ以外にはアクセスできないようにする
    ini_set('open_basedir', $htmlCacheDir);
    // キャッシュファイルを読み込んで終了する
    require $htmlCacheFilePath;
    die;
}

require $systemPathPrefix . '/vendor/autoload.php';

try {
    $app = require $systemPathPrefix . '/microengine/app.php';
} catch (\Throwable $e) {
    if (isset($displayErrorDetails) && $displayErrorDetails) {
        $content = '<p>The application could not run because of the following error: </p>'
            . '<main><article><h2>Detail</h2>'
            . '<div><strong>Message:</strong> ' . $e->getMessage() . '</div>'
            . '<div><strong>File:</strong> ' . $e->getFile() . '</div>'
            . '<div><strong>Line:</strong> ' . $e->getLine() . '</div></article>'
            . '<article><h2>Trace</h2><pre>' . $e->getTraceAsString() . '</pre></article></main>';
    } else {
        $content = '<p>The application could not run.</p>';
    }
    exit_error($content);
}

// Run Application
$app->run();
