<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Auth;

use Carbon\Carbon;
use MicroEngine\Admin\Models\User;
use Psr\Container\ContainerInterface;
use MicroEngine\Admin\Scopes\UserScope;

class Auth
{
    /**
     * ログイン済みフラグを保存するセッションキー
     */
    const AUTHENTICATED_KEY = 'authenticated';
    /**
     * ログインユーザーIDを保存するセッションキー
     */
    const USER_ID_KEY = 'loginUserId';
    /**
     * ログイン後のリダイレクトするためのパスを保存するセッションキー
     */
    const REQUESTED_PATH_KEY = 'requestedPath';
    /**
     * ログイン画面ルート名
     */
    const LOGIN_ROUTE = 'admin.auth.login';
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * ログインユーザーオブジェクト
     * @var User
     */
    protected $user;
    /**
     * リダイレクト先
     * 認証成功時にセッションを再生成するので一時的に保持する
     * @var string
     */
    protected $redirectTo;
    /**
     * 認証結果
     * @var boolean
     */
    protected $authenticationResult;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * ログイン認証試行
     *
     * @param string $loginId
     * @param string $password
     * @return bool
     */
    public function attempt(string $loginId, string $password): bool
    {
        // ログインIDに該当するユーザーを取得する
        // 普段はパスワードを除外するために UserScope を使っているが、認証に必要なためスコープなしで取得する
        $user = User::withoutGlobalScope(UserScope::class)->where('login_id', $loginId)->first();

        // 取得したユーザーのパスワードが一致することを確認する
        if ($user !== null && strlen($user->password) && !$user->isLocked() && password_verify($password, $user->password)) {
            // ログイン成功
            // パスワードにnullをセットする
            $user->password = null;
            // ログインに成功したユーザーをプロパティにセットする
            $this->user = $user;

            // リダイレクト設定
            if (isset($_SESSION[self::REQUESTED_PATH_KEY])) {
                $this->redirectTo = $_SESSION[self::REQUESTED_PATH_KEY];
            }

            // セッション情報を破棄
            $_SESSION = [];
            session_destroy();
            session_start();

            // セッションIDを再生成する
            session_regenerate_id(true);

            // セッションに認証情報を登録する
            $_SESSION[self::AUTHENTICATED_KEY] = true;
            $_SESSION[self::USER_ID_KEY] = $user->id;

            if ($user->failure_count) {
                // カウントがあれば、ゼロリセットする
                $this->c->get('userManager')->resetFailure($user->id);
            }

            // ログを出力する
            $this->c->get('logger')->notice('ログインに成功しました。', [
                'login_id' => $user->login_id,
                'name' => $user->name,
                'role' => $user->role,
            ]);

            $this->authenticationResult = true;

            return true;
        }

        // ログイン失敗
        $this->authenticationResult = false;
        // ロックアウト情報更新
        if ($user !== null && !$user->isLocked()) {
            // ロック状態で失敗日時を更新するとロックアウト期間が延びるので、ロック中の場合は更新しない。
            if ($user->lockPeriodElapsed()) {
                // ロック対象期間を過ぎたあとの失敗の場合は 0 リセットする
                $user->failure_count = 0;
            }
            $user->failure_count++;
            $user->failed_at = Carbon::now();
            $user->save();

            if ($user->failure_count === config('admin.lockout_limit', 10)) {
                // 失敗回数が制限値に到達した場合はログを出力する
                $this->c->get('logger')->notice('ユーザーをロックアウトしました。', [
                    'login_id' => $user->login_id,
                    'name' => $user->name,
                    'role' => $user->role,
                    'count' => $user->failure_count,
                ]);
            }
        }

        return false;
    }

    /**
     * ログイン済みであることを確認する
     *
     * @return bool
     */
    public function authenticated(): bool
    {
        if (gettype($this->authenticationResult) === 'boolean') {
            // 認証結果を取得済みの場合はその値を返す。
            return $this->authenticationResult;
        }

        if (isset($_SESSION[self::AUTHENTICATED_KEY]) && $_SESSION[self::AUTHENTICATED_KEY] === true) {
            // ログイン済み
            if (empty($this->user)) {
                try {
                    $this->user = User::find($_SESSION[self::USER_ID_KEY]);
                } catch (\PDOException $e) {
                    // ログイン状態でインストーラーを実行した場合にファイルを作成する際にログが取られる。
                    // そのタイミングではDBができていないのでエラーになる。
                    // その場合があるので例外は握りつぶす
                }

                if ($this->user == null) {
                    // logout() 内でこのメソッド（authenticated()）が呼ばれるので、その前に結果をセットする
                    $this->authenticationResult = false;
                    // ユーザーが見つからない場合はログアウト処理をする。
                    $this->logout();

                    // 認証失敗
                    return false;
                }
            }
            $this->authenticationResult = true;

            // 認証成功
            return true;
        }
        // 未ログイン
        $this->authenticationResult = false;

        // 認証失敗
        return false;
    }

    /**
     * ログイン後にリダイレクトするためのパスを登録する
     *
     * @param string $path
     */
    public function setRequestedPath(string $path)
    {
        $_SESSION[self::REQUESTED_PATH_KEY] = $path;
    }

    /**
     * ログアウト
     */
    public function logout()
    {
        $context = [];
        if ($this->user) {
            $context['login_id'] = $this->user->login_id;
            $context['name'] = $this->user->name;
            $context['role'] = $this->user->role;
        }

        // ログを書き出す
        $this->c->get('logger')->notice('ログアウトしました。', $context);
        $this->clearLoginSession();
    }

    /**
     * リダイレクト先のパスを返す
     *
     * @return string
     */
    public function getRedirectTo()
    {
        return $this->redirectTo;
    }

    /**
     * ログインユーザーを返す
     *
     * @return User
     */
    public function getUser(): User
    {
        return $this->user;
    }

    /**
     * ログインユーザーの id カラム値を返す
     *
     * @return int
     */
    public function getUserId(): int
    {
        if (!$this->user) {
            // ログインユーザーがいない場合は 0 を返す
            return 0;
        }
        return $this->user->id;
    }

    /**
     * ログイン済みかどうかを返す
     * 事前に authenticated() メソッドを叩いている必要がある。ミドルウェアでは isLogin() を使わずに authenticated() を使う
     *
     * @return bool
     */
    public function isLogin(): bool
    {
        return $this->user !== null;
    }

    /**
     * ログインセッション情報を消去する
     */
    protected function clearLoginSession()
    {
        // セッション変数を空にする
        $_SESSION = [];

        // クライアントに不要なクッキーが残ることを避けるためにクッキーを削除する。
        if (ini_get('session.use_cookies')) {
            $params = session_get_cookie_params();
            setcookie(session_name(), '', time() - 42000,
                $params['path'],
                $params['domain'],
                $params['secure'],
                true
            );
        }

        // セッション情報を削除する
        session_destroy();
    }
}
