<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Admin\Forms\ConfigForm;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class ConfigController extends Controller
{
    /**
     * ConfigController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->c->get('userManager')->required('config');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $configs = config('settings.config_files');

        return $this->view->render(
            $response,
            'admin/views/config/index.twig',
            compact('configs')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ConfigForm();
        $filename = $args['filename'];
        $configService = $this->c->get('configService');

        // 編集が許可されたファイルであることを確認
        if (!$configService->isEditable($filename)) {
            $this->flash->addMessage('error', '指定されたファイルを開くことができません。');
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.config.index'))->withStatus(302);
        }

        $content = $configService->getContent($filename);
        $form->setInputValue([
            'filename' => $filename,
            'content' => $content,
        ]);
        $label = $configService->getLabel($filename);
        $codeEditor = $this->c->get('codeEditor');
        $mode = $codeEditor->getMode($filename);

        return $this->view->render(
            $response,
            'admin/views/config/edit.twig',
            compact('form', 'label', 'mode')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ConfigForm();
        $filename = $args['filename'];
        $configService = $this->c->get('configService');

        // 編集が許可されたファイルであることを確認
        if (!$configService->isEditable($filename)) {
            $this->flash->addMessage('error', '指定されたファイルを開くことができません。');
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.config.index'))->withStatus(302);
        }

        $form->setInputValue($request->getParsedBody());
        $label = $configService->getLabel($filename);

        // バリデーション
        if ($form->validate()) {
            $configService->saveFile($filename, $form->get('content'));
            $this->flash->addMessage('success', '設定ファイルを保存しました。ファイル：' . $label . ' ( ' . $filename . ' )');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.config.index'))->withStatus(302);
        } else {
            $codeEditor = $this->c->get('codeEditor');
            $mode = $codeEditor->getMode($filename);

            return $this->view->render(
                $response,
                'admin/views/config/edit.twig',
                compact('form', 'label', 'mode')
            );
        }
    }
}
