<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Controllers;

use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Admin\Models\User;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Admin\Forms\UserCreateForm;
use MicroEngine\Admin\Forms\UserUpdateForm;
use MicroEngine\Core\Controllers\Controller;
use Psr\Http\Message\ServerRequestInterface;

class UserController extends Controller
{
    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required(['user', 'user_own']);

        $pageNo = $args['page_no'] ?? 1;
        $query = User::orderBy('id');

        $userCapability = $this->c->get('userManager')->hasCapability('user');
        if (!$userCapability) {
            // userを持っていなければ、ログインユーザーのみに絞り込む
            $user = $this->c->get('auth')->getUser();
            $query->where('id', $user->id);
        }
        $pagination = $query->pagination($pageNo);
        $pagination->setRouteInfo($this->c, 'admin.user.index');
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/index.twig',
            compact('pagination', 'roles', 'userCapability')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required('user');

        $form = new UserCreateForm();
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/create.twig',
            compact('form', 'roles')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws RuleQuashException
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required('user');

        $form = new UserCreateForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            $user = $this->c->get('userManager')->create($form->getInputValues());
            $this->flash->addMessage('success', "ユーザーを作成しました。ユーザー名：{$user->name} ( {$user->login_id} )");

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.user.index'))->withStatus(302);
        } else {
            $roles = array_unnest(config('settings.roles', []), 'label');

            return $this->view->render(
                $response,
                'admin/views/user/create.twig',
                compact('form', 'roles')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required(['user', 'user_own']);
        $this->c->get('userManager')->canEditOtherUser($args['id']);

        $form = new UserUpdateForm($args['id']);
        $user = User::findOrFail($args['id']);
        $form->setInputValue($user);
        $roles = array_unnest(config('settings.roles', []), 'label');

        return $this->view->render(
            $response,
            'admin/views/user/edit.twig',
            compact('form', 'user', 'roles')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required(['user', 'user_own']);
        $this->c->get('userManager')->canEditOtherUser($args['id']);

        $form = new UserUpdateForm($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            $user = $this->c->get('userManager')->update($args['id'], $form->getInputValues());
            $this->flash->addMessage('success', "ユーザーを更新しました。ユーザー名：{$user->name} ( {$user->login_id} )");

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.user.index'))->withStatus(302);
        } else {
            $user = User::findOrFail($args['id']);
            $roles = array_unnest(config('settings.roles', []), 'label');

            return $this->view->render(
                $response,
                'admin/views/user/edit.twig',
                compact('form', 'user', 'roles')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function unlock(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required('user');

        $user = $this->c->get('userManager')->resetFailure($args['id']);
        $this->flash->addMessage('success', "ユーザーのロックを解除しました。ユーザー名：{$user->name} ( $user->login_id )");
        $this->c->get('logger')->notice('ユーザーのロックアウトを解除しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
            'role' => $user->role,
        ]);
        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.user.index'))->withStatus(302);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $this->c->get('userManager')->required('user');
        $this->c->get('userManager')->canEditOtherUser($args['id']);

        $user = $this->c->get('userManager')->destroy($args['id']);
        $this->flash->addMessage('success', "ユーザーを削除しました。ユーザー名：{$user->name} ( {$user->login_id} )");

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.user.index'))->withStatus(302);
    }
}
