<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Forms;

use Slim\Psr7\UploadedFile;
use Rakit\Validation\ErrorBag;
use MicroEngine\Core\Forms\Form;
use Psr\Container\ContainerInterface;

class LicenseForm extends Form
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var array
     */
    protected $inputs = [
        'license' => [
            'label' => 'ライセンスファイル',
            'value' => '',
            'rule' => '',
        ],
    ];

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * @return bool
     */
    public function validate(): bool
    {
        foreach ($this->inputs as $name => $input) {
            // valueにアップロードファイルオプジェクトが入っている場合はライセンスを検証する
            if (is_a($input['value'], 'Slim\Psr7\UploadedFile')) {
                $this->validateLicense($input['value']);
            }
        }
        // errorBag が null かどうかでエラー有無を確認する
        return ($this->errorBag === null);
    }

    /**
     * ライセンス検証
     *
     * @param UploadedFile $uploadedFile
     * @return bool
     */
    protected function validateLicense(UploadedFile $uploadedFile): bool
    {
        // エラー確認
        if ($uploadedFile->getError() !== UPLOAD_ERR_OK) {
            $this->error('license', 'license', 'ファイルのアップロードに失敗しました。');

            return false;
        }

        // ファイルサイズ確認
        $size = $uploadedFile->getSize();
        if ($size === 0) {
            $this->error('license', 'license', 'アップロードされたファイルは空です。');

            return false;
        }
        if ($size >= 2048) {
            $this->error('license', 'license', 'アップロードされたファイルのサイズが2KBを超えています。');

            return false;
        }

        // アップロードされたファイルの内容を確認
        $encryptedLicense = $uploadedFile->getStream()->getContents();
        // ファイルを復号できることを確認
        $decryptedLicense = $this->c->get('license')->decryptLicense($encryptedLicense);

        // ライセンスの復号に失敗すると null が入っているのでそれを確認する
        if ($decryptedLicense === null) {
            $this->error('license', 'license', 'ライセンスの復号に失敗しました。');

            return false;
        }

        return true;
    }

    /**
     * エラー登録
     *
     * @param string $key
     * @param string $rule
     * @param string $message
     */
    protected function addError(string $key, string $rule, string $message)
    {
        if ($this->errorBag === null) {
            // null の場合は ErrorBag オブジェクトを作成する
            $this->errorBag = new ErrorBag();
        }
        // エラーを追加する
        $this->errorBag->add($key, $rule, $message);
    }
}
