<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Admin\Managers;

use MicroEngine\Admin\Models\User;
use Psr\Container\ContainerInterface;
use MicroEngine\Core\Exception\MeException;

class UserManager
{
    /**
     * @var string スーパーユーザーのロール
     */
    const SUPER_USER_ROLE = 'administrator';

    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * ユーザー作成
     *
     * @param array $data
     * @return User
     */
    public function create(array $data): User
    {
        if (isset($data['password']) && strlen($data['password'])) {
            $data['password'] = password_hash($data['password'], PASSWORD_DEFAULT);
        }
        // デフォルトポートレット追加
        $data['portlets'] = config('admin.default_portlets', []);

        // ユーザーインスタンス作成
        $user = new User($data);
        // ロールは複数代入できないので別途セットして更新する。
        $user->role = $data['role'];
        $user->save();

        $this->c->get('logger')->notice('ユーザーを作成しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
            'role' => $user->role,
        ]);
        return $user;
    }

    /**
     * ユーザー更新
     *
     * @param int $id
     * @param array $data
     * @return User
     */
    public function update(int $id, array $data): User
    {
        $user = User::findOrFail($id);

        if (isset($data['password']) && strlen($data['password'])) {
            $data['password'] = password_hash($data['password'], PASSWORD_DEFAULT);
        } else {
            // パスワードが入力されていないときは更新しない
            unset($data['password']);
        }
        // administrator から変更する場合
        if ($user->role === self::SUPER_USER_ROLE && isset($data['role']) && $data['role'] !== self::SUPER_USER_ROLE) {
            $administratorCount = User::where('role', self::SUPER_USER_ROLE)->count();
            if ($administratorCount <= 1) {
                // administrator が最後の一人に場合は変更できない
                unset($data['role']);
            }
        }

        // 複数属性をまとめてセットする
        $user->fill($data);
        // ログインユーザーを取得
        $loginUser = $this->c->get('auth')->getUser();
        if ((int)$loginUser->id !== (int)$id) {
            // ログインユーザー自身は以外の場合はroleをセットする
            $user->role = $data['role'];
        }
        $user->save();

        $this->c->get('logger')->notice('ユーザーを更新しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
            'role' => $user->role,
        ]);
        return $user;
    }

    /**
     * ユーザー削除
     *
     * @param int $id
     * @return User
     * @throws \Exception
     */
    public function destroy(int $id): User
    {
        $user = User::findOrFail($id);

        // ログインユーザーを取得
        $loginUser = $this->c->get('auth')->getUser();
        if ($loginUser->login_id == $user->login_id) {
            throw new MeException('ユーザー自身を削除することはできません。');
        }

        $user->delete();

        $this->c->get('logger')->notice('ユーザーを削除しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
            'role' => $user->role,
        ]);
        return $user;
    }

    /**
     * ユーザーのポートレット設定を更新
     *
     * @param int $id
     * @param array $portlets
     * @return User
     */
    public function updatePortlets(int $id, array $portlets): User
    {
        $user = User::findOrFail($id);

        // ポートレット設定をセットする
        $user->portlets = $portlets;
        // 保存する
        $user->save();

        $this->c->get('logger')->notice('ユーザーのポートレット設定を更新しました。', [
            'login_id' => $user->login_id,
            'name' => $user->name,
        ]);
        return $user;
    }

    /**
     * 失敗カウンタをリセットする
     *
     * @param int $id
     * @return User
     */
    public function resetFailure(int $id): User
    {
        $user = User::findOrFail($id);
        $user->failure_count = 0;
        $user->failed_at = null;
        $user->save();

        return $user;
    }

    /**
     * capabilityを持っていることを要求する
     *
     * @param string|array $capabilities
     * @throws \Exception
     */
    public function required($capabilities)
    {
        if (!$this->hasCapability($capabilities)) {
            // capabilityを持っていない場合は例外にする
            throw new MeException('ログイン中のユーザーはこの機能を利用することができません。');
        }
    }

    /**
     * capabilityを持っているかどうか確認
     *
     * @param string|array $capabilities
     * @return bool
     */
    public function hasCapability($capabilities)
    {
        // 文字列が渡された場合は配列に変換する
        if (!is_array($capabilities)) {
            $capabilities = [$capabilities];
        }

        // ログインユーザーを取得
        $user = $this->c->get('auth')->getUser();
        // ユーザーに指定されたロールの設定を取得
        $role = config('settings.roles.' . $user->role, []);

        // 指定された capabilities の一つでも持っていることを確認する
        $capable = false;
        foreach ($capabilities as $capability) {
            if (isset($role['capability'][$capability]) && $role['capability'][$capability]) {
                $capable = true;
                break;
            }
        }
        return $capable;
    }

    public function canEditOtherUser($id)
    {
        if (!$this->hasCapability('user')) {
            // userを持っていなければ、編集対象がログインユーザーであることを確認
            $user = $this->c->get('auth')->getUser();
            if ((int)$user->id !== (int)$id) {
                throw new MeException('このユーザーを編集・削除することはできません。');
            }
        }
    }

    /**
     * ログインユーザーのロール設定を返す
     *
     * @return array
     */
    public function loginUserRoleConfig(): array
    {
        // ログインユーザーを取得
        $user = $this->c->get('auth')->getUser();
        // ユーザーに指定されたロールの設定を取得
        $role = config('settings.roles.' . $user->role, []);

        return $role;
    }
}
