<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use GuzzleHttp\Psr7\LazyOpenStream;
use MicroEngine\Cms\Forms\BackupForm;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Core\Support\ConnectionKeeper;

class BackupController extends Controller
{
    /**
     * BackupController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->c->get('userManager')->required('backup');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // バックアップファイルのIteratorを取得
        $backupsIterator = $this->c->get('backup')->backups();
        // 配列に変換して逆順に並び替える
        $backups = array_reverse(iterator_to_array($backupsIterator));

        return $this->view->render(
            $response,
            'cms/views/backup/index.twig',
            compact('backups')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        return $this->view->render(
            $response,
            'cms/views/backup/create.twig'
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // 処理開始時間を記録
        $startTs = microtime(true);

        $keeper = new ConnectionKeeper();
        $keeper->init();
        $keeper->flush('<p>バックアップ作成を開始しました。</p>');

        // バックアップファイル作成
        $this->c->get('backup')->create($keeper);

        // 処理終了時間を記録
        $time = round(microtime(true) - $startTs, 2);
        $keeper->flush('<p>バックアップ作成が完了しました。処理時間：' . $time . ' 秒</p></body>');

        return $response;
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // バックアップファイル名
        $filename = $args['filename'] ?? null;
        // バックアップファイルの情報取得
        $info = $this->c->get('backup')->info($filename);
        $schemaVersion = config('settings.schema_version');

        return $this->view->render(
            $response,
            'cms/views/backup/info.twig',
            compact('info', 'filename', 'schemaVersion')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function restore(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // バックアップファイル名
        $filename = $args['filename'] ?? null;

        // 処理開始時間を記録
        $startTs = microtime(true);

        $keeper = new ConnectionKeeper();
        $keeper->init();
        $keeper->flush('<p>リストアを開始しました。</p>');

        // リストア実行
        $this->c->get('backup')->restore($filename, $keeper);

        // 処理終了時間を記録
        $time = round(microtime(true) - $startTs, 2);
        $keeper->flush('<p>リストアが完了しました。処理時間：' . $time . ' 秒</p></body>');

        return $response;
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function download(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $filename = $args['filename'] ?? null;

        $fullPath = $this->c->get('backup')->fullPath($filename);

        if ($filename !== null && !empty($fullPath)) {
            $newStream = new LazyOpenStream($fullPath, 'r');
            // slimのレスポンスにストリームを渡す
            $newResponse = $response->withBody($newStream);

            return $newResponse
                ->withHeader('Content-Disposition', "attachment; filename*=UTF-8''" . rawurlencode($filename))
                ->withHeader('Content-type', 'application/octet-stream');
        } else {
            $this->flash->addMessage('error', '指定されたファイルが見つかりません。ファイル名：' . $filename);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.backup.index'))->withStatus(302);
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws RuleQuashException
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new BackupForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $this->c->get('backup')->destroy($form->getInputValues());
            $this->flash->addMessage('success', 'ファイルを削除しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.backup.index'))->withStatus(302);
        } else {
            return $response->withStatus(400)->write('ファイルを削除できません。');
        }
    }
}
