<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Slim\App;
use Carbon\Carbon;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Cms\Models\Calendar;
use MicroEngine\Cms\Forms\CalendarForm;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Cms\Forms\CalendarBulkForm;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Cms\Services\ContentService;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Cms\Services\CalendarManager;
use MicroEngine\Cms\Calendar\MonthlyCalendar;

class CalendarController extends Controller
{
    /**
     * @var ContentService
     */
    protected $contentService;
    /**
     * @var CalendarManager
     */
    protected $calendarManager;

    /**
     * CalendarController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->contentService = $this->c->get('contentService');
        $this->calendarManager = $this->c->get('calendarManager');
        // ライセンスチェック
        $this->c->get('license')->hasLicense();
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_unset('calendar');

        return $this->view->render(
            $response,
            'cms/views/calendar/index.twig'
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function listing(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('calendar', $request);

        $type = $args['type'];
        $date = $args['date'] ?? date('Y-m-01');

        // パラメータの日付を元に、その年月の1日を指定する
        $date = Carbon::parse($date)->format('Y-m-01');

        $config = content_config($type, 'calendar');
        $calendar = new MonthlyCalendar();
        $calendar->pull($type, $date, 1, false);

        return $this->view->render(
            $response,
            'cms/views/calendar/listing.twig',
            compact('type', 'config', 'date', 'calendar')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $date = $args['date'];
        $config = content_config($type, 'calendar');

        $form = new CalendarForm($this->c);
        $form->setConfig($config);
        $form->setInitialValues();
        $form->setInputValue(['date' => $date]);
        $form->set('edit_mode', 'create');

        return $this->view->render(
            $response,
            'cms/views/calendar/create.twig',
            compact('type', 'config', 'form', 'date')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function copy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $date = $calendar->date;
        unset($calendar->id);
        $calendar->date = Carbon::parse($calendar->date)->addDay()->format('Y-m-d');
        $form = new CalendarForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($calendar);
        $form->set('edit_mode', 'copy');

        return $this->view->render(
            $response,
            'cms/views/calendar/create.twig',
            compact('type', 'config', 'form', 'date')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'calendar');
        $form = new CalendarForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $calendar = $this->calendarManager->create($values);
            $this->flash->addMessage('success', 'カレンダーを作成しました。日付：' . $calendar->date);

            if ($form->get('edit_mode') === 'copy') {
                // 複製した場合はhistoryに関係なく新コンテンツの情報画面に遷移させる。
                return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.info', ['id' => $calendar->id]))->withStatus(302);
            }
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.listing', ['type' => $type, 'date' => $calendar->date]))->withStatus(302);
        } else {
            $date = $form->date ?? date('Y-m-d');

            return $this->view->render(
                $response,
                'cms/views/calendar/create.twig',
                compact('type', 'config', 'form', 'date')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function preview(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = content_config($type, 'calendar');
        $form = new CalendarForm($this->c);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());
        $site = $this->c->get('site');
        $site->setPreview(true);

        // タイプ別プレビューテンプレートのパス
        $template = '@site/calendar/preview/' . $type . '.twig';
        if (!$this->view->getLoader()->exists($template)) {
            // タイプ別のテンプレートが用意されていない場合は汎用テンプレートを指定する。
            $template = 'cms/views/base/preview.twig';
        }

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $calendar = new Calendar($values);
            $site->pushContent($calendar, true);

            return $this->view->render(
                $response,
                $template,
                compact('calendar', 'config')
            );
        } else {
            return $this->view->render(
                $response,
                'cms/views/base/previewError.twig',
                compact('form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'] ?? null;
        $editMode = $args['edit_mode'] ?? 'edit';
        $revisionId = $args['revision_id'] ?? null;
        if ($revisionId) {
            $calendar = $this->calendarManager->revision($id, $revisionId);
        } else {
            $calendar = Calendar::withoutGlobalScopes()->findOrFail($id);
        }
        // 公開設定が-5の場合は、ごみ箱に入ったコンテンツなので、編集モードを trash にする。
        if ($calendar->visibility == ContentService::VISIBILITY_TRASH) {
            $editMode = 'trash';
            // ごみ箱内のコンテンツは公開をセットする
            $calendar->visibility = ContentService::VISIBILITY_PUBLIC;
        }
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $form = new CalendarForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($calendar);
        $form->set('edit_mode', $editMode);
        $form->set('revision_id', $revisionId);

        return $this->view->render(
            $response,
            'cms/views/calendar/edit.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $form = new CalendarForm($this->c);
        $form->setId($args['id']);
        $form->setConfig($config);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $calendar = $this->calendarManager->update($args['id'], $values);
            $this->flash->addMessage('success', 'カレンダーを更新しました。日付：' . $calendar->date);

            if (history_has('calendar')) {
                return $response->withHeader('location', history('calendar'))->withStatus(302);
            }
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.listing', ['type' => $type, 'date' => $calendar->date]))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/calendar/edit.twig',
                compact('type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('calendar', $request);

        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $revisions = $this->calendarManager->revisionList($calendar);

        return $this->view->render(
            $response,
            'cms/views/calendar/info.twig',
            compact('calendar', 'type', 'config', 'revisions')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];
        $editMode = $args['edit_mode'] ?? null;
        $revisionId = $args['revision_id'] ?? null;

        $calendar = $this->calendarManager->destroy($id, $revisionId);
        if ($editMode === 'revision') {
            $message = '履歴データを削除しました。';
        } elseif ($editMode === 'draft') {
            $message = '下書きを削除しました。';
        } else {
            $message = 'カレンダーをごみ箱に移動しました。';
        }
        $this->flash->addMessage('success', $message . '日付：' . $calendar->date);

        if ($editMode !== null && history_has('calendar')) {
            return $response->withHeader('location', history('calendar'))->withStatus(302);
        }
        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.listing', ['type' => $calendar->type, 'date' => $calendar->date]))->withStatus(302);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function bulkCopy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $startDt = Carbon::parse($calendar->date)->addDay();
        $endDt = $startDt->copy()->addDay();

        $form = new CalendarBulkForm();
        $form->setId($args['id']);
        $form->setInputValue([
            'date' => $calendar->date,
            'start_date' => $startDt->format('Y-m-d'),
            'end_date' => $endDt->format('Y-m-d'),
        ]);

        return $this->view->render(
            $response,
            'cms/views/calendar/bulkCopy.twig',
            compact('type', 'config', 'form', 'calendar')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function bulkStore(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $form = new CalendarBulkForm();
        $form->setId($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $saved = $this->calendarManager->bulkCopy($calendar->id, $values['start_date'], $values['end_date'], $values);
            $this->flash->addMessage('success', 'カレンダーを複製しました。複製件数： ' . $saved . ' 件');
            $date = $calendar->date;

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.listing', ['type' => $calendar->type, 'date' => $date]))->withStatus(302);
        } else {
            $config = content_config($calendar->type, 'calendar');
            $type = $calendar->type;

            return $this->view->render(
                $response,
                'cms/views/calendar/bulkCopy.twig',
                compact('type', 'config', 'form', 'calendar')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function bulkTakeDown(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $type = $calendar->type;
        $config = content_config($type, 'calendar');
        $startDt = Carbon::parse($calendar->date);
        $endDt = $startDt->copy()->addDay();

        $form = new CalendarBulkForm();
        $form->setId($args['id']);
        $form->setInputValue([
            'date' => $calendar->date,
            'start_date' => $startDt->format('Y-m-d'),
            'end_date' => $endDt->format('Y-m-d'),
        ]);

        return $this->view->render(
            $response,
            'cms/views/calendar/bulkTakeDown.twig',
            compact('type', 'config', 'form', 'calendar')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function bulkDestroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendar = Calendar::withoutGlobalScopes()->findOrFail($args['id']);
        $form = new CalendarBulkForm();
        $form->setId($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $deleted = $this->calendarManager->bulkDestroy($calendar->id, $values['start_date'], $values['end_date'], $values);
            $this->flash->addMessage('success', 'カレンダーを削除しました。削除件数： ' . $deleted . ' 件');
            $date = $calendar->date;

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.listing', ['type' => $calendar->type, 'date' => $date]))->withStatus(302);
        } else {
            $config = content_config($calendar->type, 'calendar');
            $type = $calendar->type;

            return $this->view->render(
                $response,
                'cms/views/calendar/bulkTakeDown.twig',
                compact('type', 'config', 'form', 'calendar')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function yearly(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $date = $args['date'];

        // 1月1日を指定する
        $date = Carbon::parse($date)->format('Y-01-01');

        // １年分のデータを取得する
        $calendar = $this->calendarManager->monthlyCalendarData($type, $date, 12);

        return $this->view->render(
            $response,
            'cms/views/calendar/yearly.twig',
            compact('type', 'date', 'calendar')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function trash(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $calendarsConfig = config('calendars');
        // ごみ箱内のコンテンツを取得
        $calendars = Calendar::trashed()->orderBy('updated_at', 'desc')->get();

        return $this->view->render(
            $response,
            'cms/views/calendar/trash.twig',
            compact('calendars', 'calendarsConfig')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function forceDestroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];

        $calendar = $this->calendarManager->forceDestroy($id);
        $this->flash->addMessage('success', 'カレンダーをごみ箱から削除しました。日付：' . $calendar->date);

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.trash'))->withStatus(302);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function emptyTrash(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        // ごみ箱を空にして、削除したコンテンツ数を取得
        $count = $this->calendarManager->emptyTrash();
        $this->flash->addMessage('success', 'ごみ箱を空にしました。削除カレンダー数：' . $count);

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.calendar.trash'))->withStatus(302);
    }
}
