<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Cms\Models\File;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Cms\Forms\DirCreateForm;
use MicroEngine\Cms\Forms\DirUpdateForm;
use MicroEngine\Cms\Forms\FileUpdateForm;
use MicroEngine\Cms\Managers\FileManager;
use MicroEngine\Cms\Forms\FileUploadsForm;
use MicroEngine\Core\Exception\MeException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class FileController extends Controller
{
    /**
     * @var FileManager
     */
    protected $fileManager;

    /**
     * FileController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        $this->fileManager = $this->c->get('fileManager');
        // ライセンスチェック
        $this->c->get('license')->hasLicense();
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws MeException
     */
    public function manager(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $sortArg = $args['sort'] ?? 'updated_at';
        $sort = valid_parameter($sortArg, ['filename', 'updated_at', 'size']);
        $orderArg = $args['order'] ?? 'desc';
        $order = valid_parameter($orderArg, ['desc', 'asc']);
        $dir = $request->getQueryParams()['dir'] ?? '/';
        $currentMonthDir = $this->fileManager->currentMonthDir();
        // 現在ディレクトリを呼ばれた場合は、ディレクトリを作成する。
        if ($dir === $currentMonthDir) {
            $this->fileManager->createCurrentDir();
        }

        $files = $this->fileManager->dirs($dir, null, $sort, $order);
        $dirPaths = $this->fileManager->dirPaths($dir);
        $sortOrder = $this->c->get('util')->sortOrderClass($sort, $order, ['filename', 'updated_at', 'size']);

        return $this->view->render(
            $response,
            'cms/views/file/manager.twig',
            compact('dir', 'files', 'dirPaths', 'currentMonthDir', 'sortOrder')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws RuleQuashException
     * @throws MeException
     */
    public function uploads(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new FileUploadsForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $uploadedFiles = $request->getUploadedFiles();
            $result = $this->fileManager->registerUploadedFiles($uploadedFiles, $form->get('dir'));
            $response->getBody()->write(json_encode($result));

            return $response->withHeader('Content-Type', 'application/json');
        }
        $response->getBody()->write(json_encode([]));

        return $response->withHeader('Content-Type', 'application/json');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new FileUpdateForm($args['id']);
        $file = File::findOrFail($args['id']);
        $form->setInputValue($file);

        return $this->view->render(
            $response,
            'cms/views/file/edit.twig',
            compact('form', 'file')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new FileUpdateForm($args['id']);
        $file = File::findOrFail($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $dir = $this->fileManager->updateDir($args['id'], $form->getInputValues());
            $this->flash->addMessage('success', 'ファイルを更新しました。ファイル：' . $file->filename);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $dir->dir]))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/file/edit.twig',
                compact('form', 'file')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $file = File::findOrFail($args['id']);

        return $this->view->render(
            $response,
            'cms/views/file/info.twig',
            compact('file')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws MeException
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $file = $this->fileManager->destroy($args['id']);
        $this->flash->addMessage('success', 'ファイルを削除しました。ファイル：' . $file->filename);

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $file->dir]))->withStatus(302);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function createDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $dir = $request->getQueryParams()['dir'] ?? '/';
        $form = new DirCreateForm();
        $form->setInputValue([
            'dir' => $dir
        ]);

        return $this->view->render(
            $response,
            'cms/views/file/createDir.twig',
            compact('dir', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws MeException
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function storeDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DirCreateForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $dir = $this->fileManager->createDir($form->getInputValues());
            $this->flash->addMessage('success', 'ディレクトリを作成しました。ディレクトリ：' . $dir->filename);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $dir->path()]))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/file/createDir.twig',
                compact('categories', 'type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function editDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DirUpdateForm($args['id']);
        $dir = File::findOrFail($args['id']);
        $form->setInputValue($dir);

        return $this->view->render(
            $response,
            'cms/views/file/editDir.twig',
            compact('form', 'dir')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function updateDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DirUpdateForm($args['id']);
        $dir = File::findOrFail($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $dir = $this->fileManager->updateDir($args['id'], $form->getInputValues());
            $this->flash->addMessage('success', 'ディレクトリを更新しました。ディレクトリ：' . $dir->dir);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $dir->dir]))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/file/editDir.twig',
                compact('form', 'dir')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws MeException
     */
    public function destroyDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $file = $this->fileManager->destroyDir($args['id']);
        $this->flash->addMessage('success', 'ディテクトリを削除しました。ディレクトリ：' . $file->filename);

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $file->dir]))->withStatus(302);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws MeException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function modal(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $sortArg = $args['sort'] ?? 'updated_at';
        $sort = valid_parameter($sortArg, ['filename', 'updated_at', 'size']);
        $orderArg = $args['order'] ?? 'desc';
        $order = valid_parameter($orderArg, ['desc', 'asc']);
        $queryParams = $request->getQueryParams();
        $dir = $queryParams['dir'] ?? '/';
        $mode = $queryParams['mode'] ?? null;
        $currentMonthDir = $this->fileManager->currentMonthDir();
        if ($dir === $currentMonthDir) {
            $this->fileManager->createCurrentDir();
        }

        $files = $this->fileManager->dirs($dir, $mode, $sort, $order);
        $dirPaths = $this->fileManager->dirPaths($dir);
        $sortOrder = $this->c->get('util')->sortOrderClass($sort, $order, ['filename', 'updated_at', 'size']);

        return $this->view->render(
            $response,
            'cms/views/file/modal.twig',
            compact('dir', 'files', 'dirPaths', 'currentMonthDir', 'mode', 'sortOrder')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function modalCreateDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $dir = $request->getQueryParams()['dir'] ?? '/';
        $form = new DirCreateForm();
        $form->setInputValue([
            'dir' => $dir
        ]);

        return $this->view->render(
            $response,
            'cms/views/file/modalCreateDir.twig',
            compact('dir', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws MeException
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function modalStoreDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DirCreateForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $dir = $this->fileManager->createDir($form->getInputValues());
            $this->flash->addMessage('success', 'ディレクトリを作成しました。ディレクトリ：' . $dir->dir);

            return $response->withStatus(204);
        } else {
            return $this->view->render(
                $response->withStatus(400),
                'cms/views/file/modalCreateDir.twig',
                compact('dir', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function sync(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $dir = $request->getQueryParams()['dir'] ?? '/';

        // ファイル同期実行
        $result = $this->fileManager->sync();

        // 同期結果をフラッシュメッセージで出力する
        $this->flash->addMessage('success', 'ファイルを同期しました。登録ファイル数：' . $result['add'] . '件、削除ファイル数：' . $result['delete'] . '件');

        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.file.manager', [], ['dir' => $dir]))->withStatus(302);
    }
}
