<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Cms\Models\Table;
use MicroEngine\Cms\Forms\TableForm;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;

class TableController extends Controller
{
    /**
     * TableController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);
        // ライセンスチェック
        $this->c->get('license')->hasLicense();
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function index(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_unset('table');
        $contentService = $this->c->get('contentService');
        $tables = $contentService->tableConfig();

        return $this->view->render(
            $response,
            'cms/views/table/index.twig',
            compact('tables')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function create(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = config('tables.' . $type);
        $form = new TableForm();
        $form->set('edit_mode', 'create');

        return $this->view->render(
            $response,
            'cms/views/table/create.twig',
            compact('config', 'form', 'type')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function store(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $type = $args['type'];
        $config = config('tables.' . $type);
        $form = new TableForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $tableManager = $this->c->get('tableManager');
            $table = $tableManager->create($values, $config);
            $this->flash->addMessage('success', "テーブルを作成しました。テーブル名：{$config['label']} ( {$type} )");

            if (history_has('table')) {
                return $response->withHeader('location', history('table'))->withStatus(302);
            }
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.table.index'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/table/create.twig',
                compact('config', 'form', 'type')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function edit(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'] ?? null;
        $editMode = $args['edit_mode'] ?? 'edit';
        $revisionId = $args['revision_id'] ?? null;
        if ($revisionId) {
            $table = $this->c->get('tableManager')->revision($id, $revisionId);
        } else {
            $table = Table::findOrFail($id);
        }
        $type = $table->type;
        $config = config('tables.' . $type);
        $form = new TableForm();
        $form->setId($args['id']);
        $form->setInputValue([
            'type' => $type,
            'data' => $table->jsonData($config),
        ]);
        $form->set('edit_mode', $editMode);
        $form->set('revision_id', $revisionId);

        return $this->view->render(
            $response,
            'cms/views/table/edit.twig',
            compact('type', 'config', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function update(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $table = Table::findOrFail($args['id']);
        $type = $table->type;
        $config = config('tables.' . $type);
        $form = new TableForm();
        $form->setId($args['id']);
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $values = $form->getInputValues();
            $tableManager = $this->c->get('tableManager');
            $table = $tableManager->update($args['id'], $values, $config);
            $this->flash->addMessage('success', "テーブルを更新しました。テーブル名：{$config['label']} ( {$type} )");

            if (history_has('table')) {
                return $response->withHeader('location', history('table'))->withStatus(302);
            }
            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.table.index'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/table/edit.twig',
                compact('type', 'config', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function info(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        history_set('table', $request);

        $table = Table::findOrFail($args['id']);
        $type = $table->type;
        $config = content_config($type, 'table');
        $revisions = $this->c->get('tableManager')->revisionList($table);

        return $this->view->render(
            $response,
            'cms/views/table/info.twig',
            compact('table', 'type', 'config', 'revisions')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $id = $args['id'];
        $revisionId = $args['revision_id'] ?? null;

        $table = $this->c->get('tableManager')->destroy($id, $revisionId);
        $config = config('tables.' . $table->type);
        $target = ($revisionId) ? '履歴' : 'テーブル';
        $message = "{$target}を削除しました。テーブル名：{$config['label']} ( {$table->type} )";
        $this->flash->addMessage('success', $message);

        if ($revisionId !== null && history_has('table')) {
            return $response->withHeader('location', history('table'))->withStatus(302);
        }
        return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.table.index'))->withStatus(302);
    }
}
