<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Cms\Services\Theme;
use GuzzleHttp\Psr7\LazyOpenStream;
use MicroEngine\Cms\Forms\ThemeDirForm;
use Psr\Http\Message\ResponseInterface;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Cms\Forms\ThemeUploadsForm;
use MicroEngine\Cms\Forms\ThemeFileEditForm;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Cms\Forms\ThemeFileDestroyForm;
use MicroEngine\Cms\Forms\ThemeFileDownloadsForm;

class ThemeController extends Controller
{
    /**
     * @var Theme
     */
    protected $theme;

    /**
     * ThemeController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);
        $this->theme = $this->c->get('theme');

        $this->c->get('userManager')->required('theme');
        // ライセンスチェック
        $this->c->get('license')->hasLicense();
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function manager(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $cms = config('cms');
        $dir = end_slash($request->getQueryParams()['dir'] ?? '/' . $cms['theme']);
        $files = $this->theme->dirs($dir);
        $dirPaths = $this->theme->dirPaths($dir);

        return $this->view->render(
            $response,
            'cms/views/theme/manager.twig',
            compact('dir', 'cms', 'files', 'dirPaths')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws RuleQuashException
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function uploads(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ThemeUploadsForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $uploadedFiles = $request->getUploadedFiles();
            $results = $this->theme->registerUploadedFiles($uploadedFiles, $form->get('dir'));
            $response->getBody()->write(json_encode($results));

            return $response->withHeader('Content-Type', 'application/json');
        }
        $response->getBody()->write(json_encode([]));

        return $response->withHeader('Content-Type', 'application/json');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws \Exception
     */
    public function downloads(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ThemeFileDownloadsForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $tempFilePath = $this->theme->zipArchive($form->get('paths'));
            $newStream = new LazyOpenStream($tempFilePath, 'r');
            // slimのレスポンスにストリームを渡す
            $newResponse = $response->withBody($newStream);

            // ファイル名作成
            $zipFilename = basename($form->get('dir'));
            if (empty($zipFilename)) {
                $zipFilename = 'theme';
            }
            $zipFilename .= date('_Ymd_His') . '.zip';

            return $newResponse
                ->withHeader('Content-Disposition', "attachment; filename*=UTF-8''" . rawurlencode($zipFilename))
                ->withHeader('Content-type', 'application/octet-stream');
        }
        return $response->withStatus(400)->write('Failed to create the archive file.');
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function editFile(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $path = $request->getQueryParams()['path'] ?? '';
        $dir = dirname($path);
        $content = $this->theme->getContent($path);
        $form = new ThemeFileEditForm();
        $form->setInputValue([
            'path' => $path,
            'content' => $content,
        ]);
        $codeEditor = $this->c->get('codeEditor');
        $mode = $codeEditor->getMode($path);

        return $this->view->render(
            $response,
            'cms/views/theme/editFile.twig',
            compact('path', 'form', 'dir', 'mode')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function updateFile(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ThemeFileEditForm();
        $form->setInputValue($request->getParsedBody());
        $path = $form->get('path');
        $dir = dirname($path);

        // バリデーション
        if ($form->validate()) {
            $this->theme->saveFile($path, $form->get('content'));
            $this->flash->addMessage('success', 'ファイルを保存しました。ファイル：' . $path);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.theme.manager', [], ['dir' => $dir]))->withStatus(302);
        } else {
            $codeEditor = $this->c->get('codeEditor');
            $mode = $codeEditor->getMode($path);

            return $this->view->render(
                $response,
                'cms/views/theme/editFile.twig',
                compact('path', 'form', 'dir', 'mode')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function createDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $dir = end_slash($request->getQueryParams()['dir'] ?? '/');
        $form = new ThemeDirForm();
        $form->setInputValue([
            'dir' => $dir
        ]);

        return $this->view->render(
            $response,
            'cms/views/theme/createDir.twig',
            compact('dir', 'form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     * @throws \MicroEngine\Core\Exception\MeException
     */
    public function storeDir(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ThemeDirForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $dir = $this->theme->createDir($form->getInputValues());
            $this->flash->addMessage('success', 'ディレクトリを作成しました。ディレクトリ：' . $dir);

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.theme.manager', [], ['dir' => $dir]))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'cms/views/theme/createDir.twig',
                compact('dir', 'form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws RuleQuashException
     */
    public function destroy(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new ThemeFileDestroyForm();
        $form->setInputValue($request->getParsedBody());

        // バリデーション
        if ($form->validate()) {
            $this->theme->destroy($form->getInputValues());
            $this->flash->addMessage('success', 'ファイルまたはディレクトリを削除しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.theme.manager', [], ['dir' => $form->get('dir')]))->withStatus(302);
        } else {
            return $response->withStatus(400)->write('Failed to delete the file.');
        }
    }
}
