<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Managers;

use Illuminate\Support\Arr;
use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Category;
use Psr\Container\ContainerInterface;
use Illuminate\Database\Eloquent\Collection;

class CategoryManager
{
    /**
     * @var ContainerInterface
     */
    protected $c;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * カテゴリー保存
     *
     * @param array $data
     * @return Category
     */
    public function create(array $data): Category
    {
        // 作成者・更新者にログインユーザーのIDをセットする
        $data['create_user_id'] = $data['update_user_id'] = $this->c->get('auth')->getUserId();

        $category = Category::create($data);

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $this->c->get('logger')->info('カテゴリーを作成しました。', [
            'name' => $category->name,
            'dir' => $category->dir,
        ]);
        return $category;
    }

    /**
     * カテゴリー更新
     *
     * @param int $id
     * @param array $data
     * @return Category
     */
    public function update(int $id, array $data): Category
    {
        // 更新者にログインユーザーのIDをセットする
        $data['update_user_id'] = $this->c->get('auth')->getUserId();

        $category = Category::findOrFail($id);
        $oldDir = $category->dir;
        $category->fill($data);
        $category->save();

        // ディレクトリが変更されている場合は、このカテゴリ内のページのパスを更新する。
        if ($category->dir !== $oldDir) {
            /* @var \MicroEngine\Cms\Managers\PageManager $pageManager */
            $pageManager = $this->c->get('pageManager');
            $pageManager->updateDir($category->id, $category->dir);
        }

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $this->c->get('logger')->info('カテゴリーを更新しました。', [
            'name' => $category->name,
            'dir' => $category->dir,
        ]);
        return $category;
    }

    /**
     * カテゴリー削除
     *
     * @param int $id
     * @return Category
     */
    public function destroy(int $id): Category
    {
        $category = Category::findOrFail($id);
        $category->delete();

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $this->c->get('logger')->info('カテゴリーを削除しました。', [
            'name' => $category->name,
            'dir' => $category->dir,
        ]);
        return $category;
    }

    /**
     * インデックスページの有無を更新
     *
     * @param int $id
     */
    public function updateHasIndex(int $id)
    {
        $category = Category::find($id);
        if ($category === null) {
            // 削除されたカテゴリをページが参照している場合はカテゴリが見つからないので以降の処理は行わない
            return;
        }

        $hasIndex = !Page::where('path', $category->dir)->get()->isEmpty();
        if ((boolean)$category->has_index !== $hasIndex) {
            $category->has_index = $hasIndex;
            $category->save();
        }
    }

    /**
     * 一括作成
     *
     * @param array $data
     * @return int $created 作成数
     */
    public function bulkCreate(array $data): int
    {
        $created = 0;
        // idがキーでdirが値の配列を取得
        $dirs = Category::pluck('dir', 'id');

        foreach ($data as $row) {
            // parent_dir がある場合は該当するカテゴリIDを parent_id にセットする
            if (isset($row['parent_dir'])) {
                $parentDir = $row['parent_dir'];
                if ($dirs->contains($parentDir)) {
                    $row['parent_id'] = $dirs->search($parentDir);
                } else {
                    $category = Category::where('dir', $parentDir)->first();
                    if ($category !== null) {
                        $row['parent_id'] = $category->id;
                        $dirs[$category->id] = $parentDir;
                    }
                }
            }
            $this->create($row);
            $created++;
        }
        $this->c->get('logger')->info('カテゴリーを一括作成しました。', [
            'count' => $created,
        ]);
        return $created;
    }

    /**
     * ディレクトリ設定によりページタイプを絞り込む
     *
     * @param array $pagesConfig
     * @param string $categoryDir
     * @return array
     */
    public function filterPageByDir(array $pagesConfig, string $categoryDir): array
    {
        $selected = false;
        $matched = [];

        foreach ($pagesConfig as $key => $config) {
            if (isset($config['action']) && $config['action'] == 'mapping') {
                // マッピングアクションの場合も除外する
                continue;
            }

            if (!isset($config['dir'])) {
                // dir 設定をしていない場合はフィルターしない
                array_push($matched, $key);
                continue;
            }
            $dirs = Arr::wrap($config['dir']);
            foreach ($dirs as $dir) {
                // 指定ディレクトリで始まっている場合は、一致したキーを $matched に登録する
                if ($dir === $categoryDir) {
                    array_push($matched, $key);
                    if (!$selected) {
                        $pagesConfig[$key]['selected'] = true;
                        $selected = true;
                    }
                }
            }
        }
        // 一致したキー以外は削除する。
        foreach ($pagesConfig as $key => $config) {
            // $matched に登録されていないキーは削除する
            if (array_search($key, $matched) === false) {
                unset($pagesConfig[$key]);
            }
        }
        return $pagesConfig;
    }

    /**
     * 指定ディレクトリで選択できるカテゴリー一覧を返す
     *
     * @param array $config
     * @return Collection
     */
    public function categoryOptions(array $config = []): Collection
    {
        $categories = Category::orderBy('dir', 'asc')->get();
        if (isset($config['dir'])) {
            $this->filterDir($categories, $config['dir']);
        }
        return $this->sortCategoryTree($categories);
    }

    /**
     * 指定ディレクトリでカテゴリを絞り込む
     *
     * @param Collection $categories
     * @param string|array $configDir
     */
    protected function filterDir(Collection $categories, $configDir)
    {
        $matched = [];
        $dirs = Arr::wrap($configDir);
        foreach ($dirs as $dir) {
            foreach ($categories as $key => $category) {
                // 指定ディレクトリで始まっている場合は、一致したキーを $matched に登録する
                if ($category->dir === $dir) {
                    array_push($matched, $key);
                }
            }
        }
        // 一致したキー以外は削除する。
        foreach ($categories as $key => $category) {
            // $matched に登録されていないキーは削除する
            if (array_search($key, $matched) === false) {
                unset($categories[$key]);
            }
        }
    }

    /**
     * カテゴリをツリー順に並べ替える
     *
     * @param Collection $list
     * @return Collection
     */
    public function sortCategoryTree($list): Collection
    {
        $sorted = $list->sortBy('created_at')->sortBy('display_order');
        $this->treeOrder($sorted);

        return $sorted->sortBy('tree_order');
    }

    /**
     * ツリー順に並べるためにツリーでの出現順とインデントの深さを計算する
     *
     * @param Collection $list
     * @param int $number ツリーでの出現順
     * @param int $parentId 親カテゴリのID
     * @param int $depth インデントの深さ
     */
    protected function treeOrder(Collection &$list, int &$number = 0, int $parentId = 0, int $depth = 0)
    {
        // カテゴリ毎に全カテゴリをループして子カテゴリかどうかをチェックする
        foreach ($list as &$item) {
            if ((int)$item->parent_id === $parentId) {
                // 現在カテゴリの親カテゴリIDが一致した場合
                $item->tree_order = ++$number;
                $item->depth = $depth;
                // 現在カテゴリを親に持つカテゴリを確認する
                $this->treeOrder($list, $number, $item->id, ($depth + 1));
            }
        }
    }
}
