<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Managers;

use MicroEngine\Cms\Models\Module;
use Psr\Container\ContainerInterface;
use MicroEngine\Cms\Models\ModuleRevision;

class ModuleManager
{
    use RevisionTrait;

    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var string モデルクラス名
     */
    protected $modelClass = 'MicroEngine\Cms\Models\Module';
    /**
     * @var string IDカラム名
     */
    protected $modelId = 'module_id';
    /**
     * @var string リビジョンクラス名
     */
    protected $revisionClass = 'MicroEngine\Cms\Models\ModuleRevision';
    /**
     * @var string リビジョンテーブル名
     */
    protected $revisionTable = 'module_revisions';

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * モジュール保存
     *
     * @param array $data
     * @return Module
     */
    public function create(array $data): Module
    {
        // 作成者・更新者にログインユーザーのIDをセットする
        $data['create_user_id'] = $data['update_user_id'] = $this->c->get('auth')->getUserId();

        $module = Module::create($data);

        // リビジョン保存
        $this->saveRevision($module->id, $data);

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $this->c->get('logger')->info('モジュールを作成しました。', [
            'type' => $module->type,
        ]);
        return $module;
    }

    /**
     * モジュール更新
     *
     * @param int $id
     * @param array $data
     * @return Module
     */
    public function update(int $id, array $data): Module
    {
        // 更新者にログインユーザーのIDをセットする
        $data['update_user_id'] = $this->c->get('auth')->getUserId();

        $module = Module::findOrFail($id);
        $module->fill($data);
        $module->save();

        // リビジョン保存
        $this->saveRevision($id, $data);

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $this->c->get('logger')->info('モジュールを更新しました。', [
            'type' => $module->type,
        ]);
        return $module;
    }

    /**
     * モジュール削除
     *
     * @param int $id
     * @param int|null $revisionId
     * @return Module
     */
    public function destroy(int $id, int $revisionId = null): Module
    {
        $module = Module::findOrFail($id);
        // 更新者にログインユーザーのIDをセットする
        $module->update_user_id = $this->c->get('auth')->getUserId();

        if ($revisionId) {
            // 履歴から再編集している場合は、履歴を削除する。
            $revision = ModuleRevision::findOrFail($revisionId);
            $revision->delete();

            // ログを書き出す
            $this->c->get('logger')->info('モジュールの履歴を削除しました。', [
                'type' => $module->type,
                'revision_id' => $revisionId,
            ]);
        } else {
            // 履歴から編集していない場合は、本レコードを削除する。
            $module->delete();

            // ログを書き出す
            $this->c->get('logger')->info('モジュールを削除しました。', [
                'type' => $module->type,
            ]);
        }

        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        return $module;
    }
}
