<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Models;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use MicroEngine\Cms\Scopes\PublishedScope;
use MicroEngine\Cms\Services\ContentService;

/**
 * visibility, published_at, closed_at の値により公開状態を判定する
 */
trait PublishTrait
{
    /**
     * ごみ箱に入っていない、公開か非公開のコンテンツに絞り込む
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeNotDeleted(Builder $query): Builder
    {
        return $query->withoutGlobalScope(PublishedScope::class)
            ->where('visibility', '>', ContentService::VISIBILITY_TRASH);
    }

    /**
     * ごみ箱のコンテンツだけに絞り込む
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeTrashed(Builder $query): Builder
    {
        return $query->withoutGlobalScope(PublishedScope::class)
            ->where('visibility', ContentService::VISIBILITY_TRASH)->with('updateUser');
    }

    /**
     * 最も近い未来の公開開始日時を取得
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeClosestPublishedAt(Builder $query): Builder
    {
        return $query->withoutGlobalScope(PublishedScope::class)
            ->where('visibility', ContentService::VISIBILITY_PUBLIC)
            ->where('published_at', '>=', Carbon::now());
    }

    /**
     * 最も近い未来の公開終了日時を取得
     *
     * @param Builder $query
     * @return Builder
     */
    public function scopeClosestClosedAt(Builder $query): Builder
    {
        return $query->withoutGlobalScope(PublishedScope::class)
            ->where('visibility', ContentService::VISIBILITY_PUBLIC)
            ->where('closed_at', '>=', Carbon::now());
    }

    /**
     * 公開ステータス
     *
     * @return string
     */
    public function publishStatus(): string
    {
        // 新規で下書き保存の場合
        if ((int)$this->visibility === ContentService::VISIBILITY_DRAFT) {
            return 'draft';
        }
        // 非公開設定の場合
        if ((int)$this->visibility !== ContentService::VISIBILITY_PUBLIC) {
            return 'hidden';
        }
        // 公開開始日時が設定されている場合
        // != 0 の緩やかな比較でゼロ値（'0000-00-00 00:00:00'）やnull以外が指定されていることを確認する
        if ($this->published_at != 0) {
            $dt = Carbon::parse($this->published_at);
            // 公開開始前の場合
            if (Carbon::now()->lt($dt)) {
                return 'before_published';
            }
        }
        // 公開終了日時が設定されている場合
        if ($this->closed_at != 0) {
            $dt = Carbon::parse($this->closed_at);
            if (Carbon::now()->lt($dt)) {
                // 公開終了前の場合
                return 'before_closed';
            } else {
                // 公開終了日時を過ぎている場合
                return 'closed';
            }
        }

        // 公開状態の場合
        return 'published';
    }
}
