<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use ZipArchive;
use MicroEngine\Core\Support\Zip;
use Psr\Container\ContainerInterface;
use MicroEngine\Core\Support\FileSystem;
use MicroEngine\Core\Exception\MeException;
use MicroEngine\Core\Support\ConnectionKeeper;
use Illuminate\Database\Capsule\Manager as DB;

class Backup
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(system_root('storage/backup'));
    }

    /**
     * バックアップファイルのIteratorを取得
     *
     * @return \GlobIterator
     * @throws MeException
     */
    public function backups()
    {
        return $this->fileSystem->glob('*.zip');
    }

    /**
     * ファイル情報を取得
     *
     * @param string $filename
     * @return array
     * @throws \Exception
     */
    public function info(string $filename): array
    {
        // zipアーカイブをオープン
        $zip = $this->openArchive($filename);

        // 情報ファイルの内容を取得
        $info = json_decode($zip->getFromName('backup.json'), true);
        // ファイル数を追加
        $info['count'] = $zip->numFiles;
        // zipファイルを閉じる
        $zip->close();

        // ファイルサイズを取得
        $fileStatus = $this->fileSystem->fileStatus($filename);
        $info['size'] = $fileStatus['size'];

        return $info;
    }

    /**
     * バックアップファイルのフルパスを取得する。
     * ファイルが見つからない場合は false を返す
     *
     * @param string $filename
     * @return string
     * @throws MeException
     */
    public function fullPath(string $filename): string
    {
        // バックアップファイルの Iterator を取得
        $backupsIterator = $this->backups();

        $backups = [];
        foreach ($backupsIterator as $fileInfo) {
            // ファイル名が値の配列を作成
            $backups[] = $fileInfo->getFilename();
        }

        if (in_array($filename, $backups, true) === false) {
            // バックアップファイル一覧にない場合は空文字を返す
            return '';
        }
        return $this->fileSystem->fullPath($filename);
    }

    /**
     * バックアップファイルを作成する
     *
     * @param ConnectionKeeper $keeper
     * @throws \Exception
     */
    public function create(ConnectionKeeper $keeper)
    {
        // スキーマバージョン取得
        $sv = config('settings.schema_version');
        // バックアップファイル名作成
        $backupFilename = "mecms_sv{$sv}_" . date('Ymd_Hi') . '.zip';

        // Zipオブジェクト作成
        $zip = new Zip();
        $zip->setKeeper($keeper);
        // 作成するzipファイルのパスを指定する
        $zip->setFilePath($this->fileSystem->fullPath($backupFilename));

        // system ディレクトリ内のファイルをZipアーカイブに登録
        $systemFs = new FileSystem(system_root());
        $systemPaths = $systemFs->fullPaths(['app', 'config', 'storage/license'], ['config/db.php', 'config/backup/']);
        $zip->addFiles($systemPaths, system_root());

        // public ディレクトリ内のファイルをZipアーカイブに登録
        $publicFs = new FileSystem((document_root()));
        $publicPaths = $publicFs->fullPaths(['files', 'themes', 'mecms_config.php'],
            ['themes/admin', 'themes/cms', 'themes/core', 'themes/install', 'themes/sample']);
        $zip->addFiles($publicPaths, document_root());

        // DB dump
        $tempSql = $this->dumpDb($keeper);
        // 一時ファイルのパスを取得
        $meta_data = stream_get_meta_data($tempSql);
        $tempSqlPath = $meta_data['uri'];
        // 一時ファイルをzipファイルに追加する
        $zip->addFile($tempSqlPath, 'db_dump.sql');
        fclose($tempSql);

        // バックアップ情報ファイル作成
        $loginUser = $this->c->get('auth')->getUser();
        $info = [
            'date' => date('Y-m-d H:i:s'),
            'host' => $_SERVER['HTTP_HOST'],
            'cms_version' => config('settings.version'),
            'schema_version' => config('settings.schema_version'),
            'login_id' => $loginUser->login_id,
            'user' => $loginUser->name,
        ];
        $zip->addFromString('backup.json', json_encode($info));

        $this->c->get('logger')->info('バックアップファイルを作成しました。', ['filename' => $backupFilename]);
    }

    /**
     * データベースをdumpする
     *
     * @param ConnectionKeeper $keeper
     * @return bool|resource
     */
    protected function dumpDb(ConnectionKeeper $keeper)
    {
        // sql文を書き出す一時ファイルを作成
        $tempSql = tmpfile();

        // ファイルの先頭の情報
        $date = date('Y-m-d H:i:s');
        $connection = config('db.default', 'mysql');
        $config = config('db.connections.' . $connection);
        $data = <<<EOD
/*
  MicroEngine CMS DB Backup Data
  DBMS: MySQL
  Date: {$date}
  Host: {$config['host']}
  Database: {$config['database']}
*/

SET foreign_key_checks = 0;

EOD;

        // テーブルリストを取得
        $rows = DB::select('SELECT table_name FROM information_schema.tables WHERE table_schema = database();');
        // stdClassの配列が返るのでテーブル名の配列に変換する
        $tables = [];
        foreach ($rows as $row) {
            $tables[] = $row->table_name;
        }

        // テーブル一つずつを処理する
        foreach ($tables as $table) {
            $data .= <<<EOD

/*
  Table structure for table `{$table}`
*/

DROP TABLE IF EXISTS `{$table}`;

EOD;
            // create table 文を取得
            $rows = DB::select("SHOW CREATE TABLE `{$table}`;");
            $createTable = (array)$rows[0];
            $data .= $createTable['Create Table'] . ';';

            //
            $data .= <<<EOD


/*
  Dumping data for table `{$table}`
*/

EOD;

            $rows = DB::table($table)->get()->toArray();
            $values = [];
            foreach (array_keys($rows) as $key) {
                // 配列にキャストする
                $row = (array)$rows[$key];

                $colValues = [];
                foreach ($row as $col) {
                    if ($col === null) {
                        $colValues[] = 'NULL';
                    } else {
                        $colValues[] = DB::getPdo()->quote($col);
                    }
                }
                $values[] = '(' . implode(',', $colValues) . ')';
            }
            if (count($values) > 0) {
                // データがある場合だけ書き出す。
                $data .= "\nINSERT INTO `{$table}` VALUES \n";
                $data .= implode(",\n", $values) . ";\n";
            }

            // ファイルに書き込む
            fwrite($tempSql, $data);
            $data = '';
        }


        // ファイルの最後の情報
        $data .= <<<EOD

SET foreign_key_checks = 1;

EOD;

        // ファイルに書き込む
        fwrite($tempSql, $data);

        // タイムアウトしないようにコネクションキーパーを叩く
        $keeper->keep();

        return $tempSql;
    }

    /**
     * ファイル削除
     *
     * @param array $data
     * @return bool
     * @throws MeException
     */
    public function destroy(array $data): bool
    {
        $result = $this->fileSystem->deleteFiles($data['paths']);
        if ($result) {
            $this->c->get('logger')->info('バックアップファイルを削除しました。', ['files' => $data['paths']]);
        }
        return $result;
    }

    /**
     * リストア
     * バックアップファイルを書き戻して、DBのリストアを行う。
     *
     * @param string $filename
     * @param ConnectionKeeper $keeper
     * @throws \Exception
     */
    public function restore(string $filename, ConnectionKeeper $keeper)
    {
        $this->c->get('logger')->notice('リストアを開始しました。');

        // zipアーカイブをオープンする
        $zip = $this->openArchive($filename);

        // themes内の admin, cms, core, install, sample 以外を削除する
        $publicFs = new FileSystem((document_root()));
        $files = $publicFs->dirs('themes/', ['admin', 'cms', 'core', 'install', 'sample']);
        foreach ($files as $file) {
            $keeper->keep();
            $publicFs->delete($file['path']);
        }
        // filesを削除する
        $publicFs->delete('files');

        // public内に書き出すファイルのパス
        $publicPaths = ['files/', 'themes/', 'mecms_config.php'];
        // system内に書き出すファイルのパス
        $systemPaths = ['app/', 'config/', 'storage/license/'];

        // zipファイルの内容をパスを判断して書き出す
        for ($i = 0; $i < $zip->numFiles; $i++) {
            $entryName = $zip->getNameIndex($i);
            if ($this->prefixSearch($entryName, $publicPaths)) {
                $extractResult = $zip->extractTo(document_root(), $entryName);
                if ($extractResult === false) {
                    throw new MeException('ファイル・ディレクトリの展開に失敗しました。パス：' . $entryName);
                }
            } else if ($this->prefixSearch($entryName, $systemPaths)) {
                $extractResult = $zip->extractTo(system_root(), $entryName);
                if ($extractResult === false) {
                    throw new MeException('ファイル・ディレクトリの展開に失敗しました。パス：' . $entryName);
                }
            }
            $keeper->keep();
        }

        // sqlファイルの内容を取得
        $sql = $zip->getFromName('db_dump.sql');
        // zipファイルを閉じる
        $zip->close();

        // PDO取得
        $pdo = DB::connection()->getPdo();
        // 複数のsqlを実行するために、ATTR_EMULATE_PREPARES に trueを指定する
        $pdo->setAttribute(\PDO::ATTR_EMULATE_PREPARES, true);
        // sql実行
        $keeper->keep();
        $result = DB::statement($sql);
        $keeper->keep();
        if (!$result) {
            throw new MeException('SQLの実行に失敗しました。');
        }

        // トランザクションが閉じられるので再開する
        DB::beginTransaction();

        $this->c->get('logger')->notice('リストアが完了しました。', ['filename' => $filename]);
    }

    /**
     * zipアーカイブをオープンする
     *
     * @param string $filename
     * @return ZipArchive
     * @throws MeException
     */
    protected function openArchive(string $filename): ZipArchive
    {
        // ファイルの存在確認
        $this->fileSystem->hasOrFail($filename);

        $zip = new ZipArchive();
        $openResult = $zip->open($this->fileSystem->fullPath($filename));

        if ($openResult !== true) {
            throw new MeException('ファイルのオープンに失敗しました。コード：' . $openResult);
        }

        // バックアップファイルであることを確認するために、backup.json の有無を確認する。
        if ($zip->statname('backup.json') === false) {
            throw new MeException('バックアップファイル内で backup.json ファイルを確認できませんでした。');
        }
        return $zip;
    }

    /**
     * 先頭一致確認
     *
     * $haystack 文字列に $needles配列の要素が先頭一致するかを確認する
     * @param string $haystack
     * @param array $needles
     * @return bool
     */
    protected function prefixSearch(string $haystack, array $needles)
    {
        foreach ($needles as $needle) {
            if (strpos($haystack, $needle) === 0) {
                return true;
            }
        }
        return false;
    }
}
