<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use SplFileObject;
use MicroEngine\Core\Support\FileSystem;
use Psr\Container\ContainerInterface;

class Data
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;
    /**
     * @var array
     */
    protected $types = [
        'text' => ['txt', 'text'],
        'php' => ['php'],
        'json' => ['json'],
        'ini' => ['ini'],
        'csv' => ['csv'],
        'tsv' => ['tsv'],
    ];

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(document_root('files'));
    }

    /**
     * データコンテンツ取得
     *
     * @param string $path
     * @param array $options
     * @return array|bool|mixed|null|string
     * @throws \MicroEngine\Core\Exception\MeException
     * @throws \Throwable
     */
    public function get(string $path, array $options = [])
    {
        if (!$this->fileSystem->has($path)) {
            return null;
        }
        $type = $this->dataType($path, $options);

        switch ($type) {
            case 'text':
                return $this->text($path);
                break;
            case 'php':
                return $this->php($path);
                break;
            case 'json':
                return $this->json($path, $options);
                break;
            case 'ini':
                return $this->ini($path, $options);
                break;
            case 'csv':
                return $this->csv($path, $options);
                break;
            case 'tsv':
                $options += [
                    'delimiter' => "\t",
                ];
                return $this->csv($path, $options);
                break;
        }
        return null;
    }

    /**
     * データタイプ判定
     *
     * @param string $path
     * @param array $options
     * @return string
     */
    protected function dataType(string $path, array $options = []): string
    {
        // オプションでタイプが明示されていれば、そのタイプを採用する。
        if (isset($options['type']) && in_array($options['type'], array_keys($this->types))) {
            return $options['type'];
        }

        $ext = pathinfo($path, PATHINFO_EXTENSION);
        foreach ($this->types as $type => $extensions) {
            if (in_array($ext, $extensions)) {
                return $type;
            }
        }
        return 'text';
    }

    /**
     * テキストデータ取得
     *
     * @param string $path
     * @return string
     * @throws \MicroEngine\Core\Exception\MeException
     */
    protected function text(string $path): string
    {
        $data = $this->fileSystem->get($path);

        return $data;
    }

    /**
     * PHPデータ取得
     *
     * @param string $path
     * @return mixed|null
     * @throws \Throwable
     */
    protected function php(string $path)
    {
        $data = null;
        $fullPath = $this->fileSystem->fullPath($path);
        try {
            $data = include $fullPath;
        } catch (\Throwable $e) {
            if (config('settings.debug', false)) {
                throw $e;
            }
        }
        return $data;
    }

    /**
     * JSONデータをデコードして返す
     *
     * @param string $path
     * @param array $options
     * @return mixed
     * @throws \MicroEngine\Core\Exception\MeException
     */
    protected function json(string $path, array $options = [])
    {
        $data = $this->fileSystem->get($path);
        $assoc = $options['assoc'] ?? false;

        return json_decode($data, $assoc);
    }

    /**
     * INIファイルをパースして返す
     *
     * @param string $path
     * @param array $options
     * @return array|bool
     * @throws \MicroEngine\Core\Exception\MeException
     */
    protected function ini(string $path, array $options = [])
    {
        $data = $this->fileSystem->get($path);
        $process_sections = $options['process_sections'] ?? false;
        $scanner_mode = INI_SCANNER_NORMAL;
        // 設定値があり、その文字列の定数がある場合はその定数値を採用する
        if (isset($options['scanner_mode']) && defined($options['scanner_mode'])) {
            $scanner_mode = constant($options['scanner_mode']);
        }
        return parse_ini_string($data, $process_sections, $scanner_mode);
    }

    /**
     * CSVファイルを配列にして返す
     *
     * @param string $path
     * @param array $options
     * @return array
     * @throws \MicroEngine\Core\Exception\MeException
     */
    protected function csv(string $path, array $options = []): array
    {
        $data = [];
        $options += [
            'encoding' => 'UTF-8',
            'SKIP_EMPTY' => true,
            'delimiter' => ',',
        ];

        if ($options['encoding'] === 'UTF-8') {
            $fullPath = $this->fileSystem->fullPath($path);
        } else {
            $file = $this->fileSystem->get($path);
            // 文字コードを Shift-JISからUTF-8に変換する
            $temp_data = mb_convert_encoding($file, 'UTF-8', $options['encoding']);
            // 一時ファイル作成して、UTF-8に変換されたデータを書き出す
            $temp = tmpfile();
            fwrite($temp, $temp_data);
            $meta_data = stream_get_meta_data($temp);
            $fullPath = $meta_data['uri'];
        }
        $file = new SplFileObject($fullPath);
        if ($options['SKIP_EMPTY']) {
            $flags = SplFileObject::READ_CSV | SplFileObject::READ_AHEAD | SplFileObject::SKIP_EMPTY;
        } else {
            $flags = SplFileObject::READ_CSV;
        }
        $file->setFlags($flags);
        $file->setCsvControl($options['delimiter']);
        foreach ($file as $line) {
            $data[] = $line;
        }
        return $data;
    }
}
