<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Cms\Services;

use MicroEngine\Core\Support\Zip;
use Psr\Container\ContainerInterface;
use MicroEngine\Core\Support\FileSystem;
use MicroEngine\Core\Exception\MeException;

class Theme
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var FileSystem
     */
    protected $fileSystem;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->fileSystem = new FileSystem(themes_root());
    }

    /**
     * ディレクトリパスリスト取得
     *
     * @param string $dir
     * @return array
     */
    public function dirPaths(string $dir): array
    {
        $paths = [];
        $pieces = explode('/', trim($dir, '/'));
        $path = '/';
        foreach ($pieces as $piece) {
            $path .= $piece . '/';
            $paths[$path] = $piece;
        }
        return $paths;
    }

    /**
     * ディレクトリ内のファイル一覧取得
     *
     * @param string $dir
     * @return array
     * @throws MeException
     */
    public function dirs(string $dir): array
    {
        return $this->fileSystem->dirs($dir);
    }

    /**
     * ディレクトリ作成
     *
     * @param array $data
     * @return string
     * @throws MeException
     */
    public function createDir(array $data): string
    {
        $dir = $data['dir'] . $data['filename'];
        $this->fileSystem->makeDir($dir);

        $this->c->get('logger')->info('ディレクトリを作成しました。', [
            'dir' => $dir,
        ]);
        return $dir;
    }

    /**
     * ファイル削除
     *
     * @param array $data
     * @return bool
     */
    public function destroy(array $data): bool
    {
        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $result = $this->fileSystem->deleteFiles($data['paths']);
        $this->c->get('logger')->info('ディレクトリを削除しました。', [
            'paths' => $data['paths'],
        ]);
        return $result;
    }

    /**
     * アップロードしたファイルを保存する
     *
     * @param array $uploadedFiles
     * @param string $dir
     * @return array
     * @throws MeException
     */
    public function registerUploadedFiles(array $uploadedFiles, string $dir): array
    {
        $results = [];
        $dir = end_slash($dir);

        foreach ($uploadedFiles['files'] as $uploadedFile) {
            $filename = $uploadedFile->getClientFilename();
            $result = [];
            $result['filename'] = $filename;
            $result['class'] = null;

            // エラー確認
            if ($uploadedFile->getError() !== UPLOAD_ERR_OK) {
                $result['message'] = 'エラー';
                $results[] = $result;
                continue;
            }

            // 新規登録
            $path = $dir . $filename;
            $this->fileSystem->moveUploadedFile($path, $uploadedFile);

            $result['message'] = '新規作成';
            $result['class'] = 'success';
            $results[] = $result;
            $this->c->get('logger')->info('ファイルを保存しました。', [
                'path' => $path,
            ]);
        }
        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        return $results;
    }

    /**
     * Zipアーカイブファイル作成
     *
     * @param array $paths
     * @return string 作成したアーカイブの一時ファイルのパス
     * @throws \Exception
     */
    public function zipArchive(array $paths): string
    {
        // Zipオブジェクト作成
        $zip = new Zip();

        // アーカイブに追加するファイルのフルパスを取得
        $fullPaths = $this->fileSystem->fullPaths($paths);

        // zipアーカイブにファイルを登録する
        $zip->addFiles($fullPaths, themes_root());

        // 作成した一時ファイルのパスを返す
        return $zip->getFilePath();
    }

    /**
     * ファイルの内容取得
     *
     * @param string $path
     * @return bool|string
     * @throws MeException
     */
    public function getContent(string $path)
    {
        return $this->fileSystem->get($path);
    }

    /**
     * ファイル保存
     *
     * @param string $path
     * @param $content
     * @return bool|int
     * @throws MeException
     */
    public function saveFile(string $path, $content)
    {
        // キャッシュをクリアする
        $this->c->get('htmlCache')->clear();

        $result = $this->fileSystem->put($path, $content);
        $this->c->get('logger')->info('ファイルを保存しました。', [
            'path' => $path,
        ]);
        return $result;
    }
}
