<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core;

use Monolog\Logger;
use Slim\Csrf\Guard;
use Slim\Flash\Messages;
use Slim\Views\TwigMiddleware;
use Twig\Loader\FilesystemLoader;
use Monolog\Handler\StreamHandler;
use Twig\Extension\DebugExtension;
use Monolog\Processor\WebProcessor;
use MicroEngine\Core\License\License;
use Psr\Container\ContainerInterface;
use MicroEngine\Core\Log\UserProcessor;
use Illuminate\Database\Capsule\Manager;
use Twig\Extension\StringLoaderExtension;
use MicroEngine\Core\Middleware\HeaderMiddleware;
use MicroEngine\Core\Middleware\InputValidationMiddleware;
use MicroEngine\Core\Views\{
    View, MenuManager, CoreExtension
};
use MicroEngine\Core\Support\{
    Date, ServiceProvider, Util, Base64Stream
};

class CoreServiceProvider extends ServiceProvider
{
    /**
     * サービス登録
     */
    public function register()
    {
        $c = $this->c;

        $c->set('db', function (ContainerInterface $c) {
            $capsule = new Manager;
            $connection = config('db.default', 'mysql');
            $capsule->addConnection(config('db.connections.' . $connection));
            $capsule->setAsGlobal();
            $capsule->bootEloquent();

            return $capsule;
        });
        $c->set('csrf', function (ContainerInterface $c) {
            $responseFactory = $this->app->getResponseFactory();
            $guard = new Guard($responseFactory);
            $guard->setPersistentTokenMode(true);

            return $guard;
        });
        $c->set('flash', function (ContainerInterface $c) {
            return new Messages();
        });
        $c->set('view', function (ContainerInterface $c) {
            $paths = [__DIR__ . '/../../app/themes', themes_root()];
            $loader = new FilesystemLoader($paths, themes_root());
            $view = new View($loader, [
                'cache' => __DIR__ . '/../../storage/cache/views',
                'auto_reload' => true,
                'debug' => config('settings.debug', false),
            ]);
            $loader->addPath(__DIR__ . '/../../app/resources', 'app');
            $view->addExtension(new CoreExtension($c));
            $view->addExtension(new StringLoaderExtension());
            $view->addExtension(new DebugExtension());
            return $view;
        });
        $c->set('menuManager', function (ContainerInterface $c) {
            return new MenuManager($c);
        });
        $c->set('date', function (ContainerInterface $c) {
            return new Date();
        });
        $c->set('util', function (ContainerInterface $c) {
            return new Util($c);
        });
        $c->set('logger', function (ContainerInterface $c) {
            $level = config('settings.logger.level');
            $logger = new Logger('App');
            $logger->pushProcessor(new WebProcessor(null, ['ip', 'http_method', 'url']));
            $logger->pushProcessor(new UserProcessor($c));
            $filename = current_log_filename();
            $logger->pushHandler(new StreamHandler(system_root('storage/logs/' . $filename), $level));

            return $logger;
        });
        $c->set('license', function (ContainerInterface $c) {
            Base64Stream::register();
            require_once('base64://' . __DIR__ . '/License/License.php');

            return new License($c);
        });
    }

    /**
     * サービス起動
     */
    public function boot()
    {
        // モデルを静的に実行できるように、コネクションを登録する。ここでは呼び出すだけ
        $db = $this->c->get('db');

        // アプリケーションミドルウェアに登録
        $this->app->add(function ($request, $handler) {
            // 最初のミドルウェアとして登録して、コントローラアクションに入る直前の $requestオブジェクトをコンテナに格納する。
            $this->set('request', $request);

            return $handler->handle($request);
        });
        $this->app->add(TwigMiddleware::createFromContainer($this->app));
        $this->app->add($this->c->get('csrf'));
        $this->app->add(new HeaderMiddleware($this->c));
        $this->app->add(new InputValidationMiddleware($this->c));
    }
}
