<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Forms;

use Rakit\Validation\ErrorBag;
use Rakit\Validation\Validator;
use MicroEngine\Core\Validation\MinNum;
use MicroEngine\Core\Validation\MaxNum;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Core\Validation\DirPath;
use MicroEngine\Core\Validation\Filename;
use MicroEngine\Core\Validation\MinChars;
use MicroEngine\Core\Validation\MaxChars;
use MicroEngine\Core\Validation\BetweenNum;
use MicroEngine\Core\Validation\BetweenChars;
use MicroEngine\Core\Validation\PhpSyntaxRule;
use MicroEngine\Core\Interfaces\FormInterface;
use MicroEngine\Core\Interfaces\DataInterface;
use MicroEngine\Core\Validation\UniqueCompositeRule;

abstract class Form implements FormInterface, DataInterface
{
    /**
     * @var int|null レコードID
     */
    public $id;
    /**
     * @var ErrorBag|null エラー情報
     */
    protected $errorBag;
    /**
     * @var array 入力フィールドリスト
     */
    protected $inputs = [];

    /**
     * @param int|null $id
     */
    public function __construct(int $id = null)
    {
        $this->id = $id;
        $this->setInitialInputs();
    }

    /**
     * バリデーション実行
     *
     * @throws RuleQuashException
     */
    public function validate(): bool
    {
        $validator = $this->getValidator();
        $validation = $validator->make(array_unnest($this->inputs, 'value'), array_unnest($this->inputs, 'rule'));
        $validation->setAliases(array_unnest($this->inputs, 'label'));
        $validation->validate();

        if ($validation->fails()) {
            // handling errors
            $this->errorBag = $validation->errors();

            return false;
        } else {
            $this->errorBag = null;

            return true;
        }
    }

    /**
     * 入力値をセット
     *
     * @param array $data
     */
    public function setInputValue($data)
    {
        foreach ($this->inputs as $name => $input) {
            if (isset($data[$name])) {
                if (array_key_exists('empty', $input) && strlen($data[$name]) === 0) {
                    // emptyキーがあり、値が空文字の場合は、empty設定値をセットする
                    $this->inputs[$name]['value'] = $input['empty'];
                } else {
                    $this->inputs[$name]['value'] = $data[$name];
                }
            }
        }
    }

    /**
     * 入力値を取得
     *
     * @return array
     */
    public function getInputValues(): array
    {
        return array_unnest($this->inputs, 'value');
    }

    /**
     * inputs 設定を取得
     *
     * @param string $name
     * @return array
     */
    public function getInputConfig(string $name): array
    {
        if (array_key_exists($name, $this->inputs)) {
            return $this->inputs[$name];
        }
        return [];
    }

    /**
     * 入力フィールドの値を取得
     *
     * @param string $name
     * @param mixed|null $default
     * @return mixed
     */
    public function get(string $name, $default = null)
    {
        if (array_key_exists($name, $this->inputs)) {
            return $this->inputs[$name]['value'];
        }
        return $default;
    }

    /**
     * 入力フィールドの値を設定
     *
     * @param string $name
     * @param mixed $value
     * @return mixed
     */
    public function set(string $name, $value)
    {
        if (array_key_exists($name, $this->inputs)) {
            $this->inputs[$name]['value'] = $value;
        }
    }

    /**
     * エラーメッセージ出力
     *
     * @param string $name
     * @return string
     */
    public function error(string $name): string
    {
        if ($this->errorBag !== null && $this->errorBag->has($name)) {
            return '<div class="text-danger">' . nl2br(e($this->errorBag->first($name))) . '</div>';
        }
        return '';
    }

    /**
     * エラーがあることを確認
     *
     * @return bool
     */
    public function hasError(): bool
    {
        if ($this->errorBag !== null && $this->errorBag->count() > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 全エラー情報を取得
     *
     * @return array
     */
    public function errors(): array
    {
        return $this->errorBag->all();
    }

    /**
     * selected 属性出力
     *
     * @param string $name
     * @param string $value
     * @return string
     */
    public function selected(string $name, $value): string
    {
        if ((string)$this->get($name) === (string)$value) {
            return ' selected="selected"';
        }
        return '';
    }

    /**
     * checked 属性出力
     *
     * @param string $name
     * @param mixed $value
     * @return string
     */
    public function checked(string $name, $value): string
    {
        $checked = false;
        $input = $this->get($name);
        if (is_array($input)) {
            // name[] のように複数要素を送信している場合は配列が渡される
            if (in_array($value, $input, true)) {
                $checked = true;
            }
        } else if ((string)$this->get($name) === (string)$value) {
            $checked = true;
        }
        if ($checked) {
            return ' checked="checked"';
        } else {
            return '';
        }
    }

    /**
     * @param int $id
     */
    public function setId($id)
    {
        $this->id = $id;
    }

    /**
     * @param array $inputs
     */
    public function setInputs(array $inputs)
    {
        $this->inputs = $inputs;
    }

    /**
     * 入力名と値のパラメータを返す
     *
     * @return array
     */
    public function getParams(): array
    {
        $params = [];
        foreach ($this->inputs as $inputName => $input) {
            if (isset($input) && strlen($input['value'])) {
                $params[$inputName] = $input['value'];
            }
        }
        return $params;

    }

    /**
     * Validator オブジェクト取得
     *
     * @return Validator
     * @throws RuleQuashException
     */
    protected function getValidator(): Validator
    {
        $validator = new Validator;

        // エラーメッセージをセット
        $messages = include __DIR__ . '/../resources/lang/ja/validation.php';
        $validator->setMessages($messages);

        $validator->addValidator('php', new PhpSyntaxRule());
        $validator->addValidator('unique_composite', new UniqueCompositeRule());
        $validator->addValidator('min_num', new MinNum());
        $validator->addValidator('max_num', new MaxNum());
        $validator->addValidator('between_num', new BetweenNum());
        $validator->addValidator('min_chars', new MinChars());
        $validator->addValidator('max_chars', new MaxChars());
        $validator->addValidator('between_chars', new BetweenChars());
        $validator->addValidator('filename', new Filename());
        $validator->addValidator('dir_path', new DirPath());

        return $validator;
    }

    /**
     * 初期値をセット
     */
    protected function setInitialInputs()
    {
    }
}