<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2020 MicroEngine Inc.
 */

namespace MicroEngine\Core\Handler;

use Slim\Exception\HttpException;
use Slim\Error\Renderers\HtmlErrorRenderer;
use MicroEngine\Core\Exception\CustomNotFoundException;

class CustomErrorRenderer extends HtmlErrorRenderer
{
    public function __invoke(\Throwable $exception, bool $displayErrorDetails): string
    {
        if ($exception instanceof HttpException) {
            $title = e($exception->getTitle());
            $description = e($exception->getDescription());
        } else {
            $title = 'アプリケーションエラー';
            if ($displayErrorDetails) {
                $description = 'このアプリケーションは次のエラーが発生したため実行することができません：';
            } else {
                $description = 'Webサイトでエラーが発生しました。ご不便をおかけして申し訳ありません。';
            }
        }

        if ($exception instanceof CustomNotFoundException) {
            $html = $this->renderNotFound();
            if (!empty($html)) {
                return $html;
            }
        }

        if ($displayErrorDetails) {
            // 詳細エラー表示が有効な場合
            $html = '<p>' . $description . '</p>'
                . '<main><article><h2>詳細</h2>'
                . $this->renderDetail($exception) . '</article>';

            while ($exception = $exception->getPrevious()) {
                $html .= '<article><h2>以前の例外</h2>'
                    . $this->renderDetail($exception) . '</article>';
            }
            $html .= '</main>';
        } else {
            // 詳細エラー表示が無効で、ログインしていない場合は内容は一切表示しない
            $html = '<p>' . $description . '</p>';
        }

        return <<<EOD
<!DOCTYPE html><html lang="ja"><head><meta charset="UTF-8"><title>{$title}</title><meta name="viewport" content="width=device-width,initial-scale=1">
<style>body{margin:0;padding:15px}h1{margin:0 0 15px;font-size:24px;color:#F70}article{background-color:#eee;padding:10px 15px}article+article{margin:20px 0 0}h2{border-bottom:1px solid #ccc;margin:0 0 10px;font-size:18px;padding:0 0 5px}pre{background-color:#f7f7f7;padding:10px;margin:0 0 5px;overflow-x:scroll}</style>
</head><body>
<header><h1>{$title}</h1></header>
{$html}
<a href="#" onClick="window.history.go(-1)">戻る</a>
</body></html>
EOD;
    }

    protected function renderDetail(\Throwable $exception): string
    {
        $html = sprintf('<div><strong>Type:</strong> %s</div>', get_class($exception));

        $code = $exception->getCode();
        if ($code !== null) {
            $html .= sprintf('<div><strong>Code:</strong> %s</div>', $code);
        }

        $message = $exception->getMessage();
        if ($message !== null) {
            $html .= sprintf('<div><strong>Message:</strong> %s</div>', htmlentities($message));
        }

        $file = $exception->getFile();
        if ($file !== null) {
            $html .= sprintf('<div><strong>File:</strong> %s</div>', $file);
        }

        $line = $exception->getLine();
        if ($line !== null) {
            $html .= sprintf('<div><strong>Line:</strong> %s</div>', $line);
        }

        $trace = $exception->getTraceAsString();
        if ($trace !== null) {
            $html .= '<h2>Trace</h2>';
            $html .= sprintf('<pre>%s</pre>', htmlentities($trace));
        }

        return $html;
    }

    /**
     * text/html コンテンツの not found レスポンスを返す
     *
     * @return string
     */
    protected function renderNotFound()
    {
        $title = 'ページが見つかりません / 404 Page Not Found';
        $homeUrl = '/';

        global $app;
        $view = $app->getContainer()->get('view');

        // 公開テーマに 404テンプレートがないか確認する
        $templatePath = config('settings.template_404', '@site/page/404.twig');
        if ($view->getLoader()->exists($templatePath)) {
            $page = [
                'name' => $title,
            ];
            return $view->fetch($templatePath, compact('page', 'homeUrl'));
        }

        return '';
    }
}
