<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Middleware;

use Slim\Psr7\Response;
use Slim\Routing\RouteContext;
use Slim\Exception\HttpNotFoundException;
use Slim\Exception\HttpBadRequestException;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

/**
 * 外部入力値のバリデーション
 * クエリパラメータ・POSTデータ・パスパラメータの文字エンコーディングが正しいことを確認する
 */
class InputValidationMiddleware extends Middleware
{
    /**
     * @param Request $request
     * @param RequestHandler $handler
     * @return Response
     * @throws HttpNotFoundException
     */
    public function __invoke(Request $request, RequestHandler $handler): Response
    {
        // クエリパラメータを確認
        $this->checkEncoding($request, $request->getQueryParams());
        // POSTデータを確認
        $data = $request->getParsedBody();
        if ($data !== null) {
            $this->checkEncoding($request, $data);
        }
        // パスパラメータを確認
        $route = RouteContext::fromRequest($request)->getRoute();
        if (!$route) {
            // _METHODが正しくない場合に、route情報が渡されないため、Not Found とする。
            throw new HttpNotFoundException($request);
        }

        $args = $route->getArguments();
        $this->checkEncoding($request, $args);

        return $handler->handle($request);
    }

    /**
     * 文字エンコーディングが正しいことを検証する
     *
     * @param Request $request
     * @param array $inputs
     */
    protected function checkEncoding(Request $request, array $inputs)
    {
        array_walk_recursive($inputs, function ($input) use ($request) {
            if (!mb_check_encoding($input, 'UTF-8')) {
                throw new HttpBadRequestException($request, '入力値の文字コード検証に失敗しました。');
            }
        });
    }
}
