<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Pagination;

use Illuminate\Support\Collection;
use Psr\Container\ContainerInterface;

class Pagination
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var Collection アイテムリスト
     */
    protected $items;
    /**
     * @var int 合計アイテム数
     */
    protected $total;
    /**
     * @var int 合計ページ数
     */
    protected $pageCount;
    /**
     * @var int 現在ページ番号
     */
    protected $currentNo;
    /**
     * @var int ページ番号ナビゲーションリンクの数
     */
    protected $midSize;
    /**
     * @var int ページ番号ナビゲーションの端のリンクの数
     */
    protected $endSize;
    /**
     * @var string
     */
    protected $firstPath;
    /**
     * @var string
     */
    protected $routeName;
    /**
     * @var string ページネーションの現在アイテムクラス名
     */
    protected $currentClass;
    /**
     * @var string ページネーションのリンクアイテムクラス名
     */
    protected $linkClass;
    /**
     * @var string ページネーションの省略アイテムクラス名
     */
    protected $ellipsisClass;
    /**
     * @var string ページネーションの省略テキスト(...)
     */
    protected $ellipsisText;
    /**
     * @var array パスパラメータ
     */
    protected $pathParams;
    /**
     * @var array クエリストリング
     */
    protected $queryParams;

    /**
     * @param Collection $items
     * @param int $total
     * @param int $currentNo
     * @param array $options
     */
    public function __construct(Collection $items, int $total, int $currentNo, array $options = [])
    {
        // 初期値をセットする
        $options += [
            'per_page' => 20,
            'mid_size' => 7,
            'end_size' => 1,
            'current_class' => 'active',
            'link_class' => '',
            'ellipsis_class' => 'disabled',
            'ellipsis_text' => '...',
        ];
        $this->items = $items;
        $this->total = $total;
        $this->currentNo = $currentNo;

        // 合計ページ数
        $this->pageCount = ceil($this->total / $options['per_page']);

        // 設定取得
        $this->midSize = $options['mid_size'];
        // ページネーションリンクは現在アイテムと前後移動するために最低３つにする。
        if ($this->midSize < 3) {
            $this->midSize = 3;
        }
        $this->endSize = $options['end_size'];
        $this->currentClass = $options['current_class'];
        $this->linkClass = $options['link_class'];
        $this->ellipsisClass = $options['ellipsis_class'];
        $this->ellipsisText = $options['ellipsis_text'];
    }

    /**
     * ルート情報セット
     *
     * @param ContainerInterface $c
     * @param string $routeName
     * @param array $pathParams
     * @param array $queryParams
     */
    public function setRouteInfo(ContainerInterface $c, string $routeName, $pathParams = [], $queryParams = [])
    {
        $this->c = $c;
        $this->routeName = $routeName;
        $this->pathParams = $pathParams;
        $this->queryParams = $queryParams;
    }

    /**
     * 前ページの有無を確認
     *
     * @return bool
     */
    public function hasPrev(): bool
    {
        return $this->currentNo > 1;
    }

    /**
     * 次ページの有無を確認
     *
     * @return bool
     */
    public function hasNext(): bool
    {
        return $this->currentNo < $this->pageCount;
    }

    /**
     * 前ページのURLを返す
     *
     * @return string
     */
    public function prevUrl(): string
    {
        if (!$this->hasPrev()) {
            return '#';
        }
        return $this->url($this->currentNo - 1);
    }

    /**
     * 次ページのURLを返す
     *
     * @return string
     */
    public function nextUrl(): string
    {
        if (!$this->hasNext()) {
            return '#';
        }
        return $this->url($this->currentNo + 1);
    }

    /**
     * 指定ページ番号のURL（パス部分）を返す
     *
     * @param int $pageNo
     * @return string
     */
    public function url(int $pageNo): string
    {
        global $app;

        $pathParams = $this->pathParams($pageNo);
        if ($pageNo === 1) {
            if (isset($this->firstPath)) {
                // 1ページ目はfirstPathにセットされたパスにクエリストリングをつけて返す
                $queryString = '';
                if (!empty($this->queryParams)) {
                    $queryString = '?' . http_build_query($this->queryParams);
                }
                return $this->firstPath . $queryString;
            } else {
                return $app->getRouteCollector()->getRouteParser()->urlFor($this->routeName, $pathParams, $this->queryParams);
            }
        }
        return $app->getRouteCollector()->getRouteParser()->urlFor($this->routeName, $pathParams, $this->queryParams);
    }

    /**
     * 各ページへのリンクリストを作成するための情報を返す
     *
     * @return array
     */
    public function getLinks(): array
    {
        $links = [];

        // 開始・終了ページ番号取得
        $startNo = $this->startNo();
        $stopNo = $this->stopNo($startNo);

        // 開始側のページ番号リンク作成
        if ($this->endSize) {
            foreach (range(1, $this->endSize) as $no) {
                if ($no < $startNo) {
                    $links[] = ['url' => $this->url($no), 'class' => $this->linkClass, 'text' => $no];
                }
            }
            if (($this->endSize + 1) < $startNo) {
                $links[] = ['url' => '#', 'class' => $this->ellipsisClass, 'text' => $this->ellipsisText];
            }
        }

        // ページ番号リンク作成
        foreach (range($startNo, $stopNo) as $no) {
            if ($no == $this->currentNo) {
                $links[] = ['url' => $this->url($no), 'class' => $this->currentClass, 'text' => $no];
            } else {
                $links[] = ['url' => $this->url($no), 'class' => $this->linkClass, 'text' => $no];
            }
        }

        // 終了側のページ番号リンク作成
        if ($this->endSize) {
            if (($this->pageCount - $this->endSize) > $stopNo) {
                $links[] = ['url' => '#', 'class' => $this->ellipsisClass, 'text' => $this->ellipsisText];
            }
            foreach (range(($this->pageCount - $this->endSize + 1), $this->pageCount) as $no) {
                if ($no > $stopNo) {
                    $links[] = ['url' => $this->url($no), 'class' => $this->linkClass, 'text' => $no];
                }
            }
        }
        return $links;
    }

    /**
     * 範囲外のページであることを判定する
     *
     * @return bool
     */
    public function outOfBounds(): bool
    {
        if ($this->currentNo == 1) {
            // 1ページ目は例外的に常に存在するので範囲外ではないと判定する
            return false;
        }
        if ($this->currentNo <= 0 || $this->currentNo > $this->pageCount) {
            // 0以下か合計ページ数を越えている場合は範囲外と判定する
            return true;
        }
        return false;
    }

    /**
     * @return Collection
     */
    public function getItems(): Collection
    {
        return $this->items;
    }

    /**
     * @param string $path
     */
    public function setFirstPath(string $path)
    {
        $this->firstPath = $path;
    }

    /**
     * 合計ページ数を返す
     *
     * @return int
     */
    public function getPageCount(): int
    {
        return $this->pageCount;
    }

    /**
     * 合計アイテム数を返す
     *
     * @return int
     */
    public function getTotal(): int
    {
        return $this->total;
    }

    /**
     * ページ番号リンクの開始番号を返す
     *
     * @return int
     */
    protected function startNo(): int
    {
        // 現在ページ番号を基準にして、表示リンク数の半分をオフセットする。
        $no = $this->currentNo - floor($this->midSize / 2);
        // 最終ページに近い場合は開始番号を早める
        if ($no > ($this->pageCount - $this->midSize + 1)) {
            $no = $this->pageCount - $this->midSize + 1;
        }
        // 開始番号が1より小さければ 1 をセットする
        if ($no < 1) {
            $no = 1;
        }
        return (int)$no;
    }

    /**
     * ページ番号リンクの終了番号を返す
     *
     * @param int $startNo
     * @return int
     */
    protected function stopNo(int $startNo): int
    {
        $no = $startNo + $this->midSize - 1;
        if ((int)$this->pageCount === 0) {
            // アイテムが１つもない場合でも１ページ表示するので1をセットする
            $no = 1;
        } elseif ($no > $this->pageCount) {
            $no = $this->pageCount;
        }
        return (int)$no;
    }

    /**
     * 指定したpageNoのパスパラメータを返す
     *
     * @param int $pageNo
     * @return array
     */
    protected function pathParams(int $pageNo): array
    {
        $pathParams = $this->pathParams;
        $pathParams['page_no'] = $pageNo;

        return $pathParams;
    }
}
