<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

use Carbon\Carbon;

class Date
{
    /**
     * 日付文字列をパースして Carbon オブジェクトを返す
     *
     * @param Carbon|string|null $time
     * @return Carbon
     */
    public function parse($time = null): Carbon
    {
        if ($time === null) {
            return '';
        }
        if (is_numeric($time)) {
            $time = Carbon::createFromTimestampUTC($time)->timezone(date_default_timezone_get());
        } elseif (!is_object($time)) {
            $time = Carbon::parse($time);
        }
        return $time;
    }

    /**
     * 日付フォーマット文字列を返す
     *
     * @param string|null $time
     * @param string $format
     * @return string
     */
    public function format(string $time = null, string $format = 'Y/m/d'): string
    {
        if ($time === null) {
            return '';
        }
        $dt = $this->parse($time);
        return $dt->format($format);
    }

    /**
     * ローカライズされた日付フォーマット文字列を返す
     *
     * @param string|null $time
     * @param string $format
     * @return string
     */
    public function localFormat(string $time = null, string $format = '%Y/%m/%d(%a)'): string
    {
        if ($time === null) {
            return '';
        }
        $dt = $this->parse($time);
        return $dt->formatLocalized($format);
    }

    /**
     * 設定名を指定して日付をフォーマットする
     *
     * @param string|null $time
     * @param string|null $name
     * @return string
     */
    public function formatName(string $time = null, string $name = null): string
    {
        if ($time === null) {
            return '';
        }
        $format = settings($name, 'Y/m/d');
        return $this->format($time, $format);
    }

    /**
     * 設定名を指定してローカル日付をフォーマットする
     *
     * @param string|null $time
     * @param string|null $name
     * @return string
     */
    public function localFormatName(string $time = null, string $name = null): string
    {
        if ($time === null) {
            return '';
        }
        $format = settings($name, '%Y/%m/%d(%a)');
        return $this->localFormat($time, $format);
    }

    /**
     * 日時が5分以内か当日の場合にそれぞれに対応したクラス文字列を出力する
     *
     * @param string|null $time
     * @param string $today
     * @param string $flesh
     * @return string
     */
    public function recentUpdates(string $time = null, string $today = 'today', string $flesh = 'flesh'): string
    {
        if ($time === null) {
            return '';
        }
        $dt = $this->parse($time);
        // $dtが未来ではなく、そこから5分後が未来であれば、現在から５分以内と判定する。
        if (!$dt->isFuture() && $dt->addMinutes(5)->isFuture()) {
            return $flesh;
        }
        if ($dt->isToday()) {
            return $today;
        }
        return '';
    }
}
