<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Core\Support;

use Carbon\Carbon;
use Illuminate\Support\Collection;

class TermLink
{
    /**
     * @var array
     */
    protected $links;
    /**
     * @var string
     */
    protected $year;
    /**
     * @var string
     */
    protected $month;
    /**
     * @var bool
     */
    protected $existsIndex = false;

    /**
     * 期間情報をセット
     *
     * @param Collection $termList
     * @param array $options
     */
    public function setTerm(Collection $termList, array $options)
    {
        $links = [];

        // 現在年月を取得
        $this->year = date('Y');
        $this->month = date('m');
        $currentDt = Carbon::create($this->year, $this->month);

        $startDt = null;
        $endDt = null;

        foreach ($termList as $item) {
            // 日付が空のデータはスルーする
            if ($item->year === null || $item->month === null) {
                continue;
            }

            $itemDt = Carbon::create($item->year, $item->month);

            // ループ初回の処理
            if ($startDt === null) {
                // スタート月をセット。1月を基準に、offset_month 設定分ずらす。
                $startMonth = 1 + (int)$options['offset_month'];
                $startDt = Carbon::create($item->year, $startMonth);
                $endDt = $startDt->copy()->addMonths($options['month']);

                // 範囲をアイテム日付に合わせる
                $this->adjustRange($startDt, $endDt, $itemDt, $options['month']);

                // リンク配列追加
                $links[] = $this->getLink($startDt, $endDt, $currentDt);
                continue;
            }

            // ループ２回目以降の処理
            // 現在の範囲内の場合はスルー
            if ($startDt->lte($itemDt) && $endDt->gt($itemDt)) {
                continue;
            }

            // 現在アイテムの範囲になるまで期間を進める
            while (!$endDt->gt($itemDt)) {
                $startDt->addMonths($options['month']);
                $endDt->addMonths($options['month']);

                // 空白ページ有効設定もしくは、範囲内の場合はリンクを追加
                if ($options['blank'] || $endDt->gt($itemDt)) {
                    $links[] = $this->getLink($startDt, $endDt, $currentDt);
                }
            }
        }

        // 期間リンクは昇順で処理するので、降順設定の場合は$links作成後に逆順にする
        if ($options['sort'] === 'desc') {
            $links = array_reverse($links);
        }

        // 現在期間のデータが無い場合は存在する最終期間を現在期間とする
        if (!$this->existsIndex) {
            if ($options['sort'] === 'asc') {
                // 昇順の場合は最後のデータを現在期間とする
                $lastIndex = count($links) - 1;
                if (isset($links[$lastIndex])) {
                    $this->year = $links[$lastIndex]['year'];
                    $this->month = $links[$lastIndex]['month'];
                    $links[$lastIndex]['is_index'] = true;
                }
            } else {
                // 降順の場合は最初のデータを現在期間とする
                if (isset($links[0])) {
                    $this->year = $links[0]['year'];
                    $this->month = $links[0]['month'];
                    $links[0]['is_index'] = true;
                }
            }
        }
        $this->links = $links;
    }

    /**
     * @return array
     */
    public function getLinks(): array
    {
        return $this->links;
    }

    /**
     * @return string
     */
    public function getYear(): string
    {
        return $this->year;
    }

    /**
     * @return string
     */
    public function getMonth(): string
    {
        return $this->month;
    }

    /**
     * リンク配列を作成する
     *
     * @param Carbon $startDt
     * @param Carbon $endDt
     * @param Carbon $currentDt
     * @return array
     */
    protected function getLink(Carbon $startDt, Carbon $endDt, Carbon $currentDt): array
    {
        if (!$this->existsIndex && $startDt->lte($currentDt) && $endDt->gt($currentDt)) {
            $index = true;
            $this->existsIndex = true;
            $this->year = $startDt->year;
            $this->month = $startDt->format('m');
        } else {
            $index = false;
        }
        $year = $startDt->format('Y');
        $month = $startDt->format('m');
        $day = $startDt->format('d');
        $dateString = $year . '-' . $month . '-' . $day;

        return [
            'date' => $dateString,
            'year' => $year,
            'month' => $month,
            'day' => $day,
            'is_index' => $index,
        ];
    }

    /**
     * 範囲調整
     *
     * @param Carbon $startDt
     * @param Carbon $endDt
     * @param Carbon $itemDt
     * @param int $unitMonth
     */
    protected function adjustRange(Carbon $startDt, Carbon $endDt, Carbon $itemDt, int $unitMonth)
    {
        // 範囲日付が、現在アイテム日付よりも未来の場合
        while ($startDt->gt($itemDt)) {
            // 単位月分過去に戻す
            $startDt->subMonth($unitMonth);
            $endDt->subMonth($unitMonth);
        }

        // 範囲日付が、現在アイテム日付より過去の場合
        while ($endDt->lte($itemDt)) {
            // 単位月分未来に進める
            $startDt->addMonths($unitMonth);
            $endDt->addMonths($unitMonth);
        }
    }
}
