<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

use Illuminate\Support\Arr;

if (!function_exists('config')) {
    /**
     * 設定を取得する
     *
     * @param string $key
     * @param mixed|null $default
     * @return mixed|null
     */
    function config(string $key, $default = null)
    {
        global $app;

        $c = $app->getContainer();

        // ドット区切りで先頭のキーを取得
        $filename = explode('.', $key)[0];

        // settings から始まるキーの場合は、 settings() 関数を呼び出す
        if ($filename === 'settings') {
            if ($key === 'settings') {
                return settings('', $default);
            }
            // "settings." の 9 文字分オフセットして、それ以降のキーを取り出す
            $key = substr($key, 9);
            return settings($key, $default);
        }

        // config を初期化する
        if (!$c->has('config')) {
            //$c['config'] = new \Slim\Collection();
            $c->set('config', new \Illuminate\Support\Collection());
        }
        $config = $c->get('config');

        // ドット区切りも含め対応するキーがあれば返す
        if (Arr::has($config, $key)) {
            return Arr::get($config, $key);
        }

        // 対応するキーが無い場合でも、ファイル名に対応した配列がすでにあればファイルを取得しなおす必要はないのでデフォルト値を返す。
        if ($config->has($filename)) {
            return $default;
        }

        // 設定ファイルパスを作成
        $filePath = system_root('config/' . $filename . '.php');
        // 設定ファイルがあることを確認する
        if (file_exists($filePath)) {
            $fileContent = require $filePath;
            $config[$filename] = $fileContent;
            $c->set('config', $config);
        }

        // 対応するキーがあれば返す
        if (Arr::has($config, $key)) {
            return Arr::get($config, $key);
        }
        return $default;
    }
}

if (!function_exists('settings')) {
    /**
     * settings 設定を取得する
     *
     * @param string $key
     * @param mixed|null $default
     * @return mixed|array|null
     */
    function settings(string $key = '', $default = null)
    {
        global $app;

        $c = $app->getContainer();
        $settings = $c->get('settings');

        if (empty($key)) {
            // 配列を返す
            return $settings->all();
        }

        // ドット区切りも含め対応するキーがあれば返す
        if (Arr::has($settings, $key)) {
            return Arr::get($settings, $key);
        }
        return $default;
    }
}

if (!function_exists('is_login')) {
    /**
     * ログイン済みかどうかを返す
     *
     * @return bool
     */
    function is_login(): bool
    {
        global $app;

        $c = $app->getContainer();
        return $c->get('auth')->isLogin();
    }
}

if (!function_exists('system_root')) {
    /**
     * system ディレクトリのファイルシステムパスを返す
     * 例： /filesystem/system
     *
     * @param string $path 配下のパス
     * @return string
     */
    function system_root(string $path = ''): string
    {
        return realpath(__DIR__ . '/../../../') . ($path ? '/' . $path : $path);
    }
}

if (!function_exists('document_root')) {
    /**
     * ドキュメントルートのファイルシステムパスを返す
     * 例： /filesystem/document_root
     *
     * @param string $path 配下のパス
     * @return string
     */
    function document_root(string $path = ''): string
    {
        $documentRoot = config('settings.document_root', $_SERVER['DOCUMENT_ROOT']);

        return $documentRoot . ($path ? '/' . $path : $path);
    }
}

if (!function_exists('themes')) {
    /**
     * テーマディレクトリのサイトルートパスを返します。
     * 例： /themes
     *
     * @param string $path 配下のパス
     * @return string
     */
    function themes(string $path = ''): string
    {
        $themesDir = '/' . config('settings.themes_dir', 'themes');

        return $themesDir . ($path ? '/' . ltrim($path, '/') : $path);
    }
}

if (!function_exists('themes_root')) {
    /**
     * themes ディレクトリのファイルシステムパスを返す
     * 例： /filesystem/document_root/themes
     *
     * @param string $path 配下のパス
     * @return string
     */
    function themes_root(string $path = ''): string
    {
        return document_root() . themes() . ($path ? '/' . $path : $path);
    }
}

if (!function_exists('array_unnest')) {
    /**
     * ネストした配列を元に、指定したキー名の値だけを持つ配列を返す
     *
     * 例：
     * ネストした配列
     * $items = [
     *     'html' => [
     *         'label' => 'HTML',
     *         'layout' => 'top',
     *     ],
     *     'blade' => [
     *         'label' => 'Blade',
     *         'layout' => 'master',
     *     ],
     * ],
     *
     * キーを指定して関数を実行する。
     * $result = array_unnest($items, 'label')
     *
     * 'label' の値だけを持つ配列が返される。
     * $result = [
     *     'html' => 'HTML',
     *     'blade' => 'Blade',
     * ],
     *
     * @param array $items 処理対象配列
     * @param string $columnKey 値を返したいカラムのキー名
     * @return array
     */
    function array_unnest(array $items = [], string $columnKey = ''): array
    {
        $ret = [];
        foreach ($items as $parentKey => $child) {
            if (array_key_exists($columnKey, $child)) {
                $ret[$parentKey] = $child[$columnKey];
            }
        }
        return $ret;
    }
}

if (!function_exists('end_slash')) {
    /**
     * 文字列の末尾を "/" にして返す
     *
     * @param string|null $path パス文字列
     * @return string
     */
    function end_slash(string $path = null): string
    {
        return rtrim($path, '/') . '/';
    }
}

if (!function_exists('relative_path')) {
    /**
     * サイトルートパスを相対パスに変換して返す
     *
     * @param string $from 基点のサイトルートパス
     * @param string $to 目的のサイトルートパス
     * @return string
     */
    function relative_path(string $from, string $to): string
    {
        $ret = '';

        // パスを配列に変換
        $fromArr = explode('/', ltrim($from, '/'));
        $toArr = explode('/', ltrim($to, '/'));

        // 重複するパスを削除する
        // ファイル同士の比較を避けるために、２個以上要素があることを確認する。先頭要素が同じ場合は削除する。
        while (isset($fromArr[1]) && isset($toArr[1]) && $fromArr[0] === $toArr[0]) {
            // 配列の先頭要素を削除
            array_shift($fromArr);
            array_shift($toArr);
        }

        // 基点配列の最後の要素はファイル（相当）なのでその分を引いた数を取得する
        $size = count($fromArr) - 1;
        if ($size > 0) {
            // その数ぶん'../'を追加する
            $ret .= str_repeat('../', $size);
        } else {
            $ret .= './';
        }
        // 残りのto配列を'/'で連結する
        $ret .= implode('/', $toArr);

        return $ret;
    }
}

if (!function_exists('resize')) {
    /**
     * パスにリサイズ設定用のサフィックスを追加します
     *
     * @param string $path パス文字列
     * @param string $option リサイズオプション値
     * @return string
     */
    function resize(string $path, string $option): string
    {
        if (strlen($path) && strlen($option)) {
            $pathParts = pathinfo($path);
            $dirname = $pathParts['dirname'] ?? '';
            $filename = $pathParts['filename'] ?? '';
            $extension = $pathParts['extension'] ?? '';
            // リサイズ対応拡張子の場合だけパスを書き換える。
            if (in_array($extension, config('image.resize', []))) {
                $path = $dirname . '/' . $filename . '_' . $option . '.' . $extension;
            }
        }
        return $path;
    }
}

if (!function_exists('file_type_icon')) {
    /**
     * ファイルタイプ別のアイコン用クラス文字列を返す
     * @param string $path
     * @return string
     */
    function file_type_icon(string $path): string
    {
        $icons = config('settings.icons');
        $extension = strtolower(pathinfo($path, PATHINFO_EXTENSION));
        $className = 'fa fa-fw fa-file-o';

        foreach ($icons as $name => $iconList) {
            if (in_array($extension, $iconList)) {
                $className = 'fa fa-fw fa-' . $name;
                break;
            }
        }
        return $className;
    }
}

if (!function_exists('current_log_filename')) {
    /**
     * 現在期間のログファイル名を返す
     *
     * @return string
     */
    function current_log_filename(): string
    {
        $piece = '';
        $period = config('settings.logger.period');

        if ($period === 'yearly') {
            $piece = date('-Y');
        } elseif ($period === 'monthly') {
            $piece = date('-Y-m');
        } elseif ($period === 'daily') {
            $piece = date('-Y-m-d');
        }
        return 'app' . $piece . '.log';
    }
}

if (!function_exists('history_set')) {
    /**
     * 画面遷移履歴を登録する
     *
     * @param string $name
     * @param \Psr\Http\Message\ServerRequestInterface|null $request
     */
    function history_set(string $name, Psr\Http\Message\ServerRequestInterface $request = null)
    {
        if (!isset($_SESSION['history'])) {
            // 配列を初期化
            $_SESSION['history'] = [];
        }
        if ($request) {
            $route = \Slim\Routing\RouteContext::fromRequest($request)->getRoute();
            $routeName = $route->getName();
            $args = $route->getArguments();
            $queryParams = $request->getQueryParams();

            $_SESSION['history'][$name] = [
                'routeName' => $routeName,
                'args' => $args,
                'queryParams' => $queryParams,
            ];
        }
    }
}

if (!function_exists('history_unset')) {
    /**
     * 画面遷移履歴を削除する
     *
     * @param string $name
     */
    function history_unset(string $name)
    {
        if (isset($_SESSION['history']) && isset($_SESSION['history'][$name])) {
            // 該当の履歴を削除
            unset($_SESSION['history'][$name]);
        }
    }
}

if (!function_exists('history_has')) {
    /**
     * 画面遷移履歴の有無を確認する
     *
     * @param string $name
     * @return bool
     */
    function history_has(string $name): bool
    {
        if (isset($_SESSION['history'][$name])) {
            return true;
        } else {
            return false;
        }
    }
}

if (!function_exists('history')) {
    /**
     * 画面遷移履歴の内容を確認する
     *
     * @param string $name
     * @return string|bool
     */
    function history(string $name)
    {
        global $app;

        if (isset($_SESSION['history'][$name])) {
            $history = $_SESSION['history'][$name];

            return $app->getRouteCollector()->getRouteParser()->urlFor($history['routeName'], $history['args'], $history['queryParams']);
        } else {
            return false;
        }
    }
}

if (!function_exists('valid_parameter')) {
    /**
     * 正しいパラメータの値を返す
     * 指定したリストに存在する正しいパラメータであることを確認して、その値を返す。
     * 一致するものがない場合は配列の先頭の値を返す
     *
     * @param string $param
     * @param array $validList
     * @return string|mixed
     */
    function valid_parameter(string $param, array $validList)
    {
        $key = array_search($param, $validList, true);
        if ($key !== false) {
            $validParam = $validList[$key];
        } else {
            $validParam = array_shift($validList);
        }
        return $validParam;
    }
}
