<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Controllers;

use Slim\App;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use Psr\Http\Message\ResponseInterface;
use MicroEngine\Core\Support\FileSystem;
use Rakit\Validation\RuleQuashException;
use MicroEngine\Core\Exception\MeException;
use Psr\Http\Message\ServerRequestInterface;
use MicroEngine\Core\Controllers\Controller;
use MicroEngine\Install\Forms\UserRegisterForm;
use MicroEngine\Install\Forms\DatabaseSetupForm;

class InstallController extends Controller
{
    /**
     * InstallController constructor.
     * @param App $app
     */
    public function __construct(App $app)
    {
        parent::__construct($app);

        // インストーラーではエラー詳細を表示する
        $settings = $this->c->get('settings');
        $settings['displayErrorDetails'] = 1;
        $this->c->set('settings', $settings);
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function start(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        return $this->view->render(
            $response,
            'install/views/install/start.twig'
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws MeException
     */
    public function eula(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $fs = new FileSystem(system_root('microengine/Install/resources'));
        $html = $fs->get('eula_mecms.html');
        $response->getBody()->write($html);

        return $response;
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function env(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $extensions = $this->c->get('installService')->checkPhpExtensions();
        $paths = $this->c->get('installService')->isWritableStorage();

        return $this->view->render(
            $response,
            'install/views/install/env.twig',
            compact('extensions', 'paths')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function createDatabase(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DatabaseSetupForm($this->c);

        return $this->view->render(
            $response,
            'install/views/install/database.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function storeDatabase(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new DatabaseSetupForm($this->c);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate() && $form->validateConnection()) {
            $config = $form->getInputValues();
            unset($config['connection']);
            $this->c->get('configService')->update('db.php', $config);

            $this->flash->addMessage('success', 'データベース設定を保存しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.install.createUser'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'install/views/install/database.twig',
                compact('form')
            );
        }
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function createUser(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new UserRegisterForm($this->c);

        return $this->view->render(
            $response,
            'install/views/install/user.twig',
            compact('form')
        );
    }

    /**
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @param array $args
     * @return ResponseInterface
     * @throws LoaderError
     * @throws RuleQuashException
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function storeUser(ServerRequestInterface $request, ResponseInterface $response, array $args): ResponseInterface
    {
        $form = new UserRegisterForm($this->c);
        $form->setInputValue($request->getParsedBody());

        // バリデーションチェック
        if ($form->validate()) {
            // DBスキーマ作成
            $this->c->get('installService')->createDatabaseSchema();
            $data = $form->getInputValues();
            // インストール時に作成するユーザーのロールはadministratorを指定する
            $data['role'] = $this->c->get('userManager')::SUPER_USER_ROLE;
            // ユーザー登録
            $this->c->get('userManager')->create($data);

            // サンプルデータ登録
            if (!empty($form->get('sample'))) {
                $data = include __DIR__ . '/../resources/db/sample.php';
                $import = $this->c->get('import');
                $import->import($data);
            }
            $this->flash->addMessage('success', 'インストールに成功しました。');
            $this->c->get('logger')->notice('インストールに成功しました。');

            return $response->withHeader('location', $this->app->getRouteCollector()->getRouteParser()->urlFor('admin.auth.login'))->withStatus(302);
        } else {
            return $this->view->render(
                $response,
                'install/views/install/user.twig',
                compact('form')
            );
        }
    }
}
