<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Install\Services;

use MicroEngine\Admin\Models\User;
use Psr\Container\ContainerInterface;
use MicroEngine\Core\Support\FileSystem;
use MicroEngine\Core\Support\SystemInfo;
use Illuminate\Database\Capsule\Manager as DB;

class InstallService
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * 必須PHPエクステンション
     * @var array
     */
    protected $requirePhpExtensions = [
        'mbstring',
        'pdo',
        'pdo_mysql',
        'OpenSSL',
        'Zip',
        'GD',
        'Imagick',
        'Tokenizer',
    ];

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
    }

    /**
     * インストール済みであることを確認
     *
     * @return bool
     */
    public function installed(): bool
    {
        // ユーザーが存在する場合はインストール済みと判定する
        if ($this->userExists()) {
            return true;
        }
        return false;
    }

    /**
     * 必須PHPエクステンションのローディング確認
     *
     * @return array $loadedExtensions
     */
    public function checkPhpExtensions(): array
    {
        $loadedExtensions = [];

        foreach ($this->requirePhpExtensions as $module) {
            $loadedExtensions[$module] = extension_loaded($module);
        }
        return $loadedExtensions;
    }

    /**
     * ファイルシステムの書き込み権限を確認
     *
     * @return array $targets
     */
    public function isWritableStorage(): array
    {
        $filesDir = config('cms.files_dir', 'files');

        // チェック対象パスリスト
        $targets = [];
        $targets[] = themes_root();
        $targets[] = document_root($filesDir);
        $targets[] = system_root('config');
        $targets[] = system_root('config/db.php');
        $targets[] = system_root('storage/cache');
        $targets[] = system_root('storage/logs');

        // フリップしてパスをキーにする
        $targets = array_flip($targets);

        // 対象の書き込み権限を確認する。
        foreach (array_keys($targets) as $target) {
            $targets[$target] = is_writable($target);
        }
        return $targets;
    }

    /**
     * データベーススキーマ作成
     */
    public function createDatabaseSchema()
    {
        $ddl = (new FileSystem(__DIR__ . '/../resources/db/'))->get('init_mysql.sql');

        $pdo = DB::connection()->getPdo();
        $pdo->setAttribute(\PDO::ATTR_EMULATE_PREPARES, true);

        $systemInfo = new SystemInfo($this->c);
        $versionStr = $systemInfo->dbmsVersion();
        $version = explode('.', $versionStr);
        $major = $version[0] ?? 0;
        $minor = $version[1] ?? 0;
        $createFulltextIndex = false;
        if ($major >= 8 || ($major == 5 && $minor >= 7)) {
            $createFulltextIndex = true;
            // MySQLのバージョンが8以上か、5の場合は5.7以降の場合はfulltextインデックスを指定する。
            $ddl .= <<<EOD
ALTER TABLE page_searches ADD FULLTEXT (`content`) WITH PARSER ngram;
ALTER TABLE collection_searches ADD FULLTEXT (`content`) WITH PARSER ngram;
EOD;
        }

        DB::statement($ddl);

        // トランザクションが閉じられるので再開する
        DB::beginTransaction();

        $this->c->get('logger')->notice('データベースを作成しました。', [
            'fulltext_index' => $createFulltextIndex,
        ]);
    }

    /**
     * ユーザーがDBに存在することを確認する
     * @return bool
     */
    protected function userExists(): bool
    {
        // テスト結果
        $result = true;

        try {
            $users = User::count();

            if ($users === 0) {
                $result = false;
            }
        } catch (\Exception $e) {
            // インストール前はテーブルが存在しないので例外が発生する
            $result = false;
        }
        return $result;
    }
}
