<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018 MicroEngine Inc.
 */

namespace MicroEngine\Site\Services;

use Slim\Views\Twig;
use Twig\Error\LoaderError;
use Twig\Error\SyntaxError;
use Twig\Error\RuntimeError;
use MicroEngine\Cms\Models\Page;
use MicroEngine\Cms\Models\Table;
use MicroEngine\Cms\Models\Module;
use MicroEngine\Cms\Services\Query;
use MicroEngine\Cms\Models\Calendar;
use MicroEngine\Cms\Models\Category;
use Psr\Container\ContainerInterface;
use MicroEngine\Cms\Models\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Collection as SupportCollection;

class Site
{
    /**
     * @var ContainerInterface
     */
    protected $c;
    /**
     * @var string
     */
    protected $type;
    /**
     * @var string
     */
    protected $id;
    /**
     * @var Model
     */
    protected $content;
    /**
     * @var array
     */
    protected $contentStack;
    /**
     * @var string フィールド名
     */
    protected $field;
    /**
     * @var array フィールド設定
     */
    protected $fieldConfig;
    /**
     * @var string 親フィールド名
     */
    protected $parentField;
    /**
     * @var int|null コンテナ配列のインデックス
     */
    protected $index;
    /**
     * @var bool プレビューモードフラグ
     */
    protected $preview = false;

    /**
     * @param ContainerInterface $c
     */
    public function __construct(ContainerInterface $c)
    {
        $this->c = $c;
        $this->contentStack = [];
    }

    /**
     * コンテンツをスタックに追加
     *
     * @param Model $content
     * @param bool $isMain
     */
    public function pushContent(Model $content, bool $isMain = false)
    {
        // スタックに追加する
        array_push($this->contentStack, $content);
        // 現在コンテンツにもセットする
        $this->content = $content;

        if ($isMain) {
            $this->type = $content->type;
            $this->id = $content->id;
        }
    }

    /**
     * コンテンツをスタックから取り除く
     *
     * @return Model
     */
    public function popContent(): Model
    {
        $content = array_pop($this->contentStack);
        $this->content = end($this->contentStack);
        is_array($this->content) && reset($this->content);

        return $content;
    }

    /**
     * 現在フィールド情報をセットする
     *
     * @param string $field
     * @param string|null $parentField
     * @param int|null $index
     */
    public function setData(string $field, string $parentField = null, int $index = null)
    {
        $this->field = $field;
        $this->parentField = $parentField;
        $this->index = $index;
    }

    /**
     * インプットID取得
     *
     * @param string $input
     * @return string
     */
    public function inputId(string $input): string
    {
        $id = $this->field . '-' . $input;
        if ($this->parentField !== null) {
            $id = $this->parentField . '-' . $this->index . '-' . $id;
        }
        return $id;
    }

    /**
     * インプット名（name属性）取得
     *
     * @param string $input
     * @param bool $multiple
     * @return string
     */
    public function inputName(string $input, bool $multiple = false): string
    {
        $name = '[' . $this->field . '][' . $input . ']';
        if ($this->parentField !== null) {
            $name = '[' . $this->parentField . '][' . $this->index . ']' . $name;
        }
        if ($multiple) {
            $name .= '[]';
        }
        return 'fields' . $name;
    }

    /**
     * インプット用属性出力
     *
     * @param string $input
     * @return string
     */
    public function inputAttr(string $input): string
    {
        $attr = '';

        // inputの属性設定を取得
        $attrSettings = $this->fieldConfig['inputs'][$input]['attr'] ?? [];

        // 属性設定を展開する
        foreach ($attrSettings as $attrName => $attrValue) {
            $attr .= ' ' . $attrName . '="' . e($attrValue) . '"';
        }

        return $attr;
    }

    /**
     * コレクションコンテンツを出力する
     *
     * @param Collection $collection
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function collection(Collection $collection): string
    {
        $type = $collection->type;
        $this->pushContent($collection);
        $config = content_config($type, 'collection');
        $template = '@site/collection/' . $type . '.twig';

        /* @var Twig $view */
        $view = $this->c->get('view');

        $rendered = $view->getEnvironment()->render(
            $template,
            compact('collection', 'config')
        );
        $this->popContent();

        return $rendered;
    }

    /**
     * モジュールコンテンツを出力する
     *
     * @param $module
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function module($module): string
    {
        if (is_string($module)) {
            // モジュール名が渡された場合
            $type = $module;
            $module = Module::where('type', $type)->first();
            if ($module === null) {
                // DBにレコードが無い場合は空のモジュールオブジェクトを作成する
                $module = new Module([
                    'type' => $type,
                    'fields' => []
                ]);
            }
        } else {
            // Module オブジェクトが渡された場合
            $type = $module->type;
        }
        $this->pushContent($module);

        $config = content_config($type, 'module');
        $template = '@site/module/' . $type . '.twig';

        /* @var Twig $view */
        $view = $this->c->get('view');

        $rendered = $view->getEnvironment()->render(
            $template,
            compact('module', 'config')
        );
        $this->popContent();

        return $rendered;
    }

    /**
     * カレンダーコンテンツを出力する
     *
     * @param Calendar $calendar
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function calendar(Calendar $calendar): string
    {
        $type = $calendar->type;
        $this->pushContent($calendar);
        $config = content_config($type, 'calendar');

        /* @var Twig $view */
        $view = $this->c->get('view');

        $templatePath = '@site/calendar/' . $type . '.twig';
        if (!$view->getLoader()->exists($templatePath)) {
            $templatePath = 'cms/views/calendar/previewItem.twig';
        }
        $rendered = $view->getEnvironment()->render(
            $templatePath,
            compact('calendar', 'config')
        );
        $this->popContent();

        return $rendered;
    }

    /**
     * フィールドを出力する
     *
     * @param string $fieldName
     * @param Model|null $content
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function field(string $fieldName, Model $content = null): string
    {
        if ($content !== null) {
            $this->pushContent($content);
        }
        $rendered = $this->renderField($fieldName);

        if ($content !== null) {
            $this->popContent();
        }
        return $rendered;
    }

    /**
     * 属性を出力する
     *
     * @param array $field
     * @param string|null $defaultClass
     * @return string
     */
    public function fieldAttr(array $field, string $defaultClass = null): string
    {
        $attr = '';

        if (isset($field['attr_id']) && strlen($field['attr_id'])) {
            $attr .= ' id="' . e($field['attr_id']) . '"';
        }
        // デフォルトクラスの指定があれば追加する
        if ($defaultClass !== null) {
            if (isset($field['attr_class']) && strlen($field['attr_class'])) {
                $field['attr_class'] = $defaultClass . ' ' . $field['attr_class'];
            } else {
                $field['attr_class'] = $defaultClass;
            }
        }
        if (isset($field['attr_class']) && strlen($field['attr_class'])) {
            $attr .= ' class="' . e($field['attr_class']) . '"';
        }
        if (isset($field['attr_style']) && strlen($field['attr_style'])) {
            $attr .= ' style="' . e($field['attr_style']) . '"';
        }
        return $attr;
    }

    /**
     * 子フィールドを出力する
     *
     * @param string $fieldName
     * @param int|null $index
     * @param string|null $parentField
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    public function childField(string $fieldName, int $index = null, string $parentField = null): string
    {
        return $this->renderField($fieldName, $index, $parentField);
    }

    /**
     * @param bool $preview
     */
    public function setPreview(bool $preview)
    {
        $this->preview = $preview;
    }

    /**
     * @return bool
     */
    public function isPreview(): bool
    {
        return $this->preview;
    }

    /**
     * data-id 属性にフィールドIDを出力する
     *
     * @param string $name
     * @return string
     */
    public function fieldDataId(string $name): string
    {
        // プレビュー時で、メインコンテンツの場合のみ data-id を出力する。
        if ($this->preview && $this->content->type === $this->type && $this->content->id === $this->id) {
            return ' data-id="' . $this->inputId($name) . '"';
        }
        return '';
    }

    /**
     * data-id 属性を出力する
     *
     * @param string $id
     * @return string
     */
    public function pageDataId(string $id): string
    {
        // プレビュー時で、メインコンテンツの場合のみ data-id を出力する。
        if ($this->preview && $this->content->type === $this->type && $this->content->id === $this->id) {
            return ' data-id="' . $id . '"';
        }
        return '';
    }

    /**
     * インプット部品の設定取得
     *
     * @param string $input
     * @param array $config
     * @return array
     */
    public function getOptions(string $input, array $config): array
    {
        $options = [];

        if (isset($config['inputs'][$input]['table'])) {
            $options = Table::type($config['inputs'][$input]['table']);
        } elseif (isset($config['inputs'][$input]['options'])) {
            $options = $config['inputs'][$input]['options'];
        }
        return $options;
    }

    /**
     * 最新N件のコンテンツを取得
     *
     * @param string $type
     * @param int $limit
     * @param string|null $baseType
     * @return SupportCollection
     * @throws \Exception
     */
    public function latest(string $type, int $limit = 3, string $baseType = null): SupportCollection
    {
        $query = new Query($this->c);

        return $query->get([
            'base_type' => $baseType,
            'type' => $type,
            'order' => 'date',
            'sort' => 'desc',
            'limit' => $limit,
        ]);
    }

    /**
     * 現在のパスを返す
     *
     * @return string
     */
    public function getPath(): string
    {
        return $this->c->get('request')->getUri()->getPath();
    }

    /**
     * パンくずナビデータを取得
     *
     * @param int|null $categoryId
     * @return array
     */
    public function breadcrumbs(int $categoryId = null): array
    {
        $breadcrumbs = [];

        // コンテンツが無い場合はパンくずデータは返さない
        if (!isset($this->contentStack[0])) {
            return $breadcrumbs;
        }

        $page = $this->contentStack[0];
        $categoryId = $categoryId ?? $page->category_id;
        $isIndex = ($page->filename === 'index');
        if (!$isIndex) {
            // インデックスではない場合は、現在ページ名をパンくずに追加する
            array_unshift($breadcrumbs, ['name' => $page->name, 'url' => '']);
        }

        while ($category = Category::find($categoryId)) {
            $categoryId = $category->parent_id;
            // インデックスページの場合は、カテゴリ名のみ追加する
            if ($isIndex) {
                array_unshift($breadcrumbs, ['name' => $category->name, 'url' => '']);
                // 現在ページの処理が終わったらフラグをfalseにして上位カテゴリではチェックしない。
                $isIndex = false;
                continue;
            }
            // インデックス有りページだけ追加する
            if ($category->has_index) {
                array_unshift($breadcrumbs, ['name' => $category->name, 'url' => $category->dir]);
            }
            if (!$category->parent_id) {
                break;
            }
        }
        return $breadcrumbs;
    }

    /**
     * サイトマップデータを取得
     *
     * @return array
     */
    public function sitemap()
    {
        $sitemap = [];

        // すべてのカテゴリを取得してカテゴリツリーに並べ替える
        $categories = $this->c->get('categoryManager')->sortCategoryTree(Category::all());
        // 親カテゴリが0のルート直下のカテゴリごとにまとめるためのインデックス番号
        $root = -1;
        foreach ($categories as $category) {
            // parent_id が 0 のルート直下のカテゴリか、初回（$root が -1）のばあいは インクリメントする
            if ($category->parent_id === 0 || $root === -1) {
                $root++;
            }
            $item = [];
            // カテゴリをセット
            $item['category'] = $category;
            // そのカテゴリ内のページを取得して pages にセット
            $item['pages'] = Page::where('category_id', $category->id)
                ->where('sitemap', 1)
                ->orderBy('display_order', 'asc')
                ->orderBy('id', 'asc')
                ->get();

            if (count($item['pages'])) {
                // ページが１件以上あればサイトマップに表示する
                $sitemap[$root][] = $item;
            }
        }
        return $sitemap;
    }

    /**
     * 現在リクエストのパスと指定されたパスが一致する場合は指定されたクラス名を返す
     *
     * @param string $path
     * @param string|null $class
     * @return string
     */
    public function isCurrent(string $path, string $class = null): string
    {
        $ret = '';
        if ($this->getPath() === $path) {
            $ret = $class ?? config('cms.is_current', 'current');
        }
        return $ret;
    }

    /**
     * 現在リクエストのディレクトリのパスと指定されたパスが一致する場合は指定されたクラス名を返す
     *
     * @param string $path
     * @param string|null $class
     * @return string
     */
    public function isCurrentDir(string $path, string $class = null): string
    {
        $ret = '';

        // dirnameは親のディレクトリを返すので、/products/ の場合は / になってしまう。その対策で、 . を追加している。
        $currentDir = dirname($this->getPath() . '.') . '/';
        if ($currentDir === end_slash($path)) {
            $ret = $class ?? config('cms.is_current_dir', 'current');
        }
        return $ret;
    }

    /**
     * 現在リクエストのパスが指定されたパスを含む（指定パス配下の）場合は指定されたクラス名を返す
     *
     * @param string $path
     * @param string|null $class
     * @return string
     */
    public function isParent(string $path, string $class = null): string
    {
        $ret = '';

        // 現在のパスが指定されたパスで始まっていることを確認する。
        if (strpos($this->getPath(), $path) === 0) {
            $ret = $class ?? config('cms.is_parent', 'current');
        }
        return $ret;
    }

    /**
     * メインコンテンツIDを返す
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * メインコンテンツのコンテンツタイプを返す
     *
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param array $fieldConfig
     */
    public function setFieldConfig(array $fieldConfig)
    {
        $this->fieldConfig = $fieldConfig;
    }

    /**
     * フィールドをレンダリングする
     *
     * @param string $fieldName
     * @param int|null $index
     * @param string|null $parentField
     * @return string
     * @throws LoaderError
     * @throws RuntimeError
     * @throws SyntaxError
     */
    protected function renderField(string $fieldName, int $index = null, string $parentField = null): string
    {
        $rendered = '';

        /**
         * @var Twig $view
         */
        $view = $this->c->get('view');

        $this->setData($fieldName, $parentField, $index);

        if ($this->content === null) {
            return $rendered;
        }
        $content = $this->content;
        $contentConfig = content_config($content->type, $content->getBaseType());
        $config = [];

        if ($parentField === null) {
            // コンテナ外のフィールド
            if (!isset($content->fields[$fieldName])) {
                return $rendered;
            }
            $field = $content->fields[$fieldName];
            if (isset($contentConfig['fields'][$fieldName])) {
                $config = $contentConfig['fields'][$fieldName];
            }
        } else {
            // コンテナ内のフィールド
            if (!isset($content->fields[$parentField][$index][$fieldName])) {
                return $rendered;
            }

            $field = $content->fields[$parentField][$index][$fieldName];
            if (isset($contentConfig['fields'][$parentField]['fields'][$fieldName])) {
                $config = $contentConfig['fields'][$parentField]['fields'][$fieldName];
            }
        }

        $template = $config['output_template'] ?? $config['type'] ?? $fieldName;
        // @site下のテンプレート有無を確認
        $templatePath = '@site/outputs/' . $template . '.twig';
        if (!$view->getLoader()->exists($templatePath)) {
            // @site下に無い場合は、cms 下のテンプレートを確認
            $templatePath = 'cms/views/outputs/' . $template . '.twig';
            if (!$view->getLoader()->exists($templatePath)) {
                // どちらにもテンプレートがなければ空を返す
                return $rendered;
            }
        }

        // フィールドをレンダリングすると $this のプロパティが書き換えられるので、その前に前後のタグを作っておく。
        $tag = $config['preview_tag'] ?? 'span';
        $renderedPrefix = $renderedSuffix = '';
        if ($this->preview && isset($config['preview_input'])) {
            $renderedPrefix = '<' . $tag . $this->fieldDataId($config['preview_input']) . '>';
            $renderedSuffix = '</' . $tag . '>';
        }

        // フィールドをレンダリングする
        $rendered .= $view->getEnvironment()->render(
            $templatePath,
            compact('field', 'config', 'fieldName')
        );

        // プレビュー時で preview_input 設定があれば data-id属性を出力する
        if ($this->preview && isset($config['preview_input'])) {
            $rendered = $renderedPrefix . $rendered . $renderedSuffix;
        }

        // コンテナ内で warp_div 指定されていれば div で包含する
        $wrapDiv = $config['wrap_div'] ?? false;
        if ($parentField !== null && $wrapDiv) {
            $rendered = '<div>' . $rendered . '</div>';
        }

        return $rendered;
    }
}
