<?php
/**
 * MicroEngine CMS
 * https://microengine.jp
 *
 * (c) 2018-2020 MicroEngine Inc.
 */

use DI\Container;
use Slim\Factory\AppFactory;

// コンテナとAppを作成
$c = new Container();
AppFactory::setContainer($c);
$app = AppFactory::create();

// settings は最初にセットする
$c->set('settings', array_merge(require(__DIR__ . '/app_settings.php'), require(__DIR__ . '/../config/settings.php')));
$settings = $c->get('settings');

// 定数定義
define('SOFTWARE_VERSION', $settings['version']);
define('SOFTWARE_COPYRIGHT_YEAR', $settings['copyright_year']);

// 詳細エラー表示設定。このファイルの読み込み元の index.php で使われるために定義する。
$displayErrorDetails = $settings['displayErrorDetails'] ?? false;

// 詳細エラー表示をしない場合は、PHP の display_errors に 0 をセットしてエラーをHTMLに出力しないようにする。
if (!$displayErrorDetails) {
    ini_set('display_errors', 0);
}

// PHP設定
foreach ($settings['ini_set'] ?? [] as $iniName => $iniValue) {
    ini_set($iniName, $iniValue);
}

// タイムゾーンの設定
date_default_timezone_set($settings['timezone'] ?? 'Asia/Tokyo');

// ロケール設定
setlocale(LC_ALL, $settings['locale'] ?? 'ja_JP.UTF-8');

// サービスプロバイダを登録する
$serviceNames = $settings['services'] ?? [];
$services = [];

// 各サービスの register 処理
foreach ($serviceNames as $serviceName) {
    $serviceProviderClass = '\\MicroEngine\\' . $serviceName . '\\' . $serviceName . 'ServiceProvider';
    $serviceProvider = new $serviceProviderClass($app);
    $serviceProvider->register();
    $services[$serviceName] = $serviceProvider;
}

// 各サービスの boot 処理
foreach ($services as $serviceProvider) {
    $serviceProvider->boot();
}

// Register routes
if (file_exists(__DIR__ . '/../app/routes.php')) {
    require __DIR__ . '/../app/routes.php';
}

// ルーティングミドルウェアを追加
$app->addRoutingMiddleware();

// HTTPメソッド上書きミドルウェアを追加
$methodOverrideMiddleware = new \Slim\Middleware\MethodOverrideMiddleware();
$app->add($methodOverrideMiddleware);

// ログ設定を取得
$logError = $settings['logErrors'] ?? true;
$logErrorDetails = $settings['logErrorDetails'] ?? false;

// エラーミドルウェアを最後に追加する
$errorMiddleware = $app->addErrorMiddleware($displayErrorDetails, $logError, $logErrorDetails);

// エラーレンダラーを登録
$myErrorHandler = new \MicroEngine\Core\Handler\CustomErrorHandler($app->getCallableResolver(), $app->getResponseFactory());
$myErrorHandler->registerErrorRenderer('text/html', \MicroEngine\Core\Handler\CustomErrorRenderer::class);
$errorMiddleware->setDefaultErrorHandler($myErrorHandler);

// アプリケーションを返す
return $app;
